 
#define DIALOGS_H

/****************************************************************

    Copyright (c) 1996-1997 Fred Pesch

    This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License as published by the Free Software 
    Foundation (version 2 of the License).

    This program is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the 
    implied warranty of MERCHANTABILITY or FITNESS FOR A 
    PARTICULAR PURPOSE.  See the GNU General Public License 
    for more details.

    For a copy of the GNU General Public License, write to the 
    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
    MA 02139, USA.

****************************************************************/

#include <Xm/Xm.h>
#include <Xm/XmAll.h>

#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include <time.h>

#include "fof.h"
#include "select.h"
#include "interface.h"
#include "fileview.h"
#include "error_handler.h"
#include "pixmap.h"
#include "offix_dnd.h"
#include "dirhistory.h"
#include "config.h"


typedef struct {
  WidgetList w_icon;
  int nb_icons;
  int nb_icons_managed;
} TDirIconList;

TDirIconList CopyDirIconList;
TDirIconList MoveDirIconList;

/* New data types */
struct CopyDirEntry {
  char *name;
  int writeable;
  int readable;
  int icon_index;
  struct CopyDirEntry *next;
};

typedef struct CopyDirEntry TCopyDirEntry;

typedef struct {
  int interactive;
  int recurse;
  int preserve_access;
  int preserve_links;
  int update;
  int hardlinks;
  int symlinks;
  int one_filesystem;
} TCopyFlags;

typedef struct {
  int interactive;
  int update;
} TMoveFlags;

typedef struct {
  int interactive;
  int recurse;
} TDeleteFlags;

typedef struct {
  int user_read;
  int user_write;
  int user_exec;
  int group_read;
  int group_write;
  int group_exec;
  int others_read;
  int others_write;
  int others_exec;
} TAttributesFlags;

typedef struct {
  int gzip;
  int absolute;
  int dereference;
  int preserve;
  int one_filesystem;
  int block_size;
  int block_size_value;
} TTarFlags;

typedef struct {
  int quality;
  int interactive;
  int recurse;
} TGZipFlags;

typedef struct {
  int size;
  int prefix;
  char *prefix_value;
} TSplitFlags;

/* External functions (avoiding circular reference) */
extern void FileCopyCommandRoutine(char *SourceDirectory, char *DestDirectory, TCopyFlags flags);
extern void FileMoveCommandRoutine(char *SourceDirectory, char *DestDirectory, TMoveFlags flags);
extern void FileDeleteCommandRoutine(char *SourceDirectory, TDeleteFlags flags);
extern void FileRenameCommandRoutine(char *SourceFile, char *Destfile);
extern void FileAttributesCommandRoutine(char *SourceFile, TAttributesFlags flags);
extern void FileCreateCommandRoutine(char *DestFile, Bool makedir);
extern void FileExecCommandRoutine(char *program, char *arguments, Bool needxterm);
extern void ArchiveTarCommandRoutine(char *DestFile, TTarFlags flags);
extern void ArchiveUnTarCommandRoutine(char *DestFile, int gunzip);
extern void ArchiveGZipCommandRoutine(TGZipFlags flags);
extern void ArchiveGUnZipCommandRoutine(TGZipFlags flags);
extern void ArchiveEncodeCommandRoutine(char *NameFile, char *DestFile);
extern void ArchiveDecodeCommandRoutine(char *DestFile);
extern void ArchiveSplitCommandRoutine(TSplitFlags flags);
extern void ArchiveUnSplitCommandRoutine(int count, char **SourceFiles, char *destfile);
extern void DropCommandRoutine(void);

extern int GoToDirRefreshRoutine(char *destdir);
extern void GoToDirNewWindow(char *dir);

/* Module variables */

TCopyDirEntry *CopyDirList;

char *SourceDirectory;
char *CurrentSelectedDirectory;

char *SourceFile;

char **SourceFiles;

Widget W_CopyTextField, W_CopyContainer;
Widget W_HardLinks, W_SymLinks;

Widget W_MoveTextField, W_MoveContainer;

Widget W_RenameSourceTextField, W_RenameDestTextField;

Widget W_Init[8];

/***************************************************************************
******************************* MISC ROUTINES ******************************
****************************************************************************/

char *GetTypeString(PRIMARY_FILE_TYPE type)
{
  switch(type) {
  case DIRECTORY :
    return("Directory");
    break;
  case SLINK_FILE :
    return("Symbolic Link");
    break;
  case SLINK_DIR :
    return("Directory Link");
    break;
  case REGULAR_FILE :
    return("Regular File");
    break;
  case EXECUTABLE_FILE :
    return("Executable File");
    break;
  case CHAR_DEVICE :
    return("Char Device");
    break;
  case BLOCK_DEVICE :
    return("Block Device");
    break;
  case FIFO :
    return("FIFO");
    break;
  case SOCKET :
    return("Socket");
    break;
  default:
    return("Unknown");
  }
}

/***************************************************************************
************************ COPY DIALOG ROUTINE *******************************
****************************************************************************/

int CreateCopyDirIcon(Arg args[], int nbargs)
{
  /* if there is no more pre-allocated buffer */
  if(CopyDirIconList.nb_icons_managed==CopyDirIconList.nb_icons) {
    CopyDirIconList.w_icon= (WidgetList) XtRealloc((char *) CopyDirIconList.w_icon, sizeof(Widget)*CopyDirIconList.nb_icons+1);
    CopyDirIconList.w_icon[CopyDirIconList.nb_icons]= XmCreateIconGadget(W_CopyContainer, "diricon", args, nbargs);
    XtManageChild(CopyDirIconList.w_icon[CopyDirIconList.nb_icons]);
    CopyDirIconList.nb_icons++;
    CopyDirIconList.nb_icons_managed++;
    return(CopyDirIconList.nb_icons-1);
  } else {
    XtSetValues(CopyDirIconList.w_icon[CopyDirIconList.nb_icons_managed], args, nbargs);
    XtManageChild(CopyDirIconList.w_icon[CopyDirIconList.nb_icons_managed]);
    CopyDirIconList.nb_icons_managed++;
    return(CopyDirIconList.nb_icons_managed-1);
  }
}

void FileCopyDeleteDirList(void)
{
  TCopyDirEntry *actual;

  /* Destroy Old Dir List */
  if(CopyDirList!=NULL) {
    actual = CopyDirList;
    while(CopyDirList!=NULL) {
      actual=CopyDirList;
      CopyDirList=CopyDirList->next;
      if(actual->name!=NULL) XtFree((char *) actual->name);
      XtFree((char *) actual);
    }
  }
  XtUnmanageChildren(CopyDirIconList.w_icon, CopyDirIconList.nb_icons_managed);
  CopyDirIconList.nb_icons_managed=0;
  
}

void FileCopyCreateDirList(void)
{
  Arg args[3];
  int nbargs;
  DIR *dir;
  struct dirent *entry;
  TCopyDirEntry *actual;
  struct stat info;
  XmString dirstring;
  int was_a_dir;
  int is_root_dir;
  int i;

  dir = opendir(CurrentSelectedDirectory);
  if(dir==NULL) {
    MessageDialog("Read Error", "Cannot open directory");
    return;
  }

  /* set textfield string */

  nbargs=0;
  XtSetArg(args[nbargs], XmNvalue, CurrentSelectedDirectory);nbargs++;
  XtSetArg(args[nbargs], XmNcursorPosition, strlen(CurrentSelectedDirectory));nbargs++;
  XtSetValues(W_CopyTextField, args, nbargs);

  /* Destroy Old Dir List */
  FileCopyDeleteDirList();

  /* Make New List */
  CopyDirList = (TCopyDirEntry *) XtMalloc(sizeof(TCopyDirEntry));
  actual = CopyDirList;
  actual->next=NULL;
  entry = readdir(dir);
  do {
    actual->name= (char *) XtMalloc(sizeof(char)*(strlen(CurrentSelectedDirectory)+strlen(entry->d_name)+2));
    strcpy(actual->name, CurrentSelectedDirectory);
    if(strcmp(actual->name,"/")==0) is_root_dir=1; else is_root_dir=0;
    if(!is_root_dir && strcmp(entry->d_name,"..")==0) {
      i=strlen(actual->name)-1;
      while(actual->name[i]!='/' && i>0) {
	actual->name[i]=(char)0;
	i--;
      }
      if(i>0) actual->name[i]=(char)0;
    } else {
      if(strcmp(actual->name,"/")!=0) strcat(actual->name, "/");
      strcat(actual->name, entry->d_name);
    }
    stat(actual->name, &info);
    /* if actual entry is a directory */
    was_a_dir=0;
    if(S_ISDIR(info.st_mode) && strcmp(entry->d_name,".")!=0 && ((is_root_dir && strcmp(entry->d_name,"..")!=0) || !is_root_dir)) {
      was_a_dir=1;
      if(access(actual->name, W_OK)==0) actual->writeable=1; else actual->writeable=0;
      if(access(actual->name, R_OK)==0) actual->readable=1; else actual->readable=0;
      /* if user can read and write in directory */
      if(actual->readable && actual->writeable) {
	dirstring = XmStringCreateSimple(entry->d_name);
	if(dirstring==NULL) FatalError("filecopy.c","FileCopyCreateDirList","Cannot create icon string");
	nbargs=0;
	XtSetArg(args[nbargs], XmNsmallIconPixmap, PIX_DIR_closed);nbargs++;
	XtSetArg(args[nbargs], XmNlabelString, dirstring);nbargs++;
	XtSetArg(args[nbargs], XmNentryParent, (Widget) NULL);nbargs++;
	actual->icon_index=CreateCopyDirIcon(args, nbargs);
	XmStringFree(dirstring);
	/* else if user can't read in directory */
      } else if(!actual->readable) {
	dirstring = XmStringCreateSimple(entry->d_name);
	if(dirstring==NULL) FatalError("filecopy.c","FileCopyCreateDirList","Cannot create icon string");
	nbargs=0;
	XtSetArg(args[nbargs], XmNsmallIconPixmap, PIX_DIR_unreadable);nbargs++;
	XtSetArg(args[nbargs], XmNlabelString, dirstring);nbargs++;
	XtSetArg(args[nbargs], XmNentryParent, (Widget) NULL);nbargs++;
	actual->icon_index=CreateCopyDirIcon(args, nbargs);
	XmStringFree(dirstring);
      }  else if(!actual->writeable) {
	dirstring = XmStringCreateSimple(entry->d_name);
	if(dirstring==NULL) FatalError("filecopy.c","FileCopyCreateDirList","Cannot create icon string");
	nbargs=0;
	XtSetArg(args[nbargs], XmNsmallIconPixmap, PIX_DIR_read_only);nbargs++;
	XtSetArg(args[nbargs], XmNlabelString, dirstring);nbargs++;
	XtSetArg(args[nbargs], XmNentryParent, (Widget) NULL);nbargs++;
	actual->icon_index=CreateCopyDirIcon(args, nbargs);
	XmStringFree(dirstring);
      }
    } else {
      XtFree(actual->name);
      actual->name=NULL;
    }
    entry = readdir(dir);
    if(entry!=NULL && was_a_dir) {
      /* Create next node */
      actual->next = (TCopyDirEntry *) XtMalloc(sizeof(TCopyDirEntry));
      actual=actual->next;
      actual->name=NULL;
      actual->icon_index=-1;
      actual->next=NULL;
    }
  } while(entry!=NULL);
  closedir(dir);
}
    

void FileCopyRescanButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtFree(CurrentSelectedDirectory);
  CurrentSelectedDirectory= XmTextGetString(W_CopyTextField);

  FileCopyCreateDirList();
}

void FileCopyOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  TCopyFlags flags;
  Widget w_temp;
  Bool set;

  XtFree(CurrentSelectedDirectory);
  FileCopyDeleteDirList();
  CurrentSelectedDirectory= XmTextGetString(W_CopyTextField);
  if(access(CurrentSelectedDirectory, W_OK)!=0) {
    MessageDialog("write error", "Cannot write destination directory");
    XtUnmanageChild(W_CopyDialog);
    XtFree(CurrentSelectedDirectory);
    XtFree(SourceDirectory);
    return;
  }
  /* get command params */

  /* interactive mode */
  w_temp=XtNameToWidget(W_CopyDialog, "*CopyInteractiveButton");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.interactive=1; else flags.interactive=0;

  /* recurse subdirectories */
  w_temp=XtNameToWidget(W_CopyDialog, "*CopyRecurseButton");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.recurse=1; else flags.recurse=0;

  /* preserve access */
  w_temp=XtNameToWidget(W_CopyDialog, "*CopyPreserveAccessButton");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.preserve_access=1; else flags.preserve_access=0;

  /* preserve access */
  w_temp=XtNameToWidget(W_CopyDialog, "*CopyPreserveLinksButton");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.preserve_links=1; else flags.preserve_links=0;

  /* update files */
  w_temp=XtNameToWidget(W_CopyDialog, "*CopyUpdateButton");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.update=1; else flags.update=0;

  /* make hard links */
  w_temp=XtNameToWidget(W_CopyDialog, "*CopyHardLinksButton");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.hardlinks=1; else flags.hardlinks=0;

  /* make symbolic links */
  w_temp=XtNameToWidget(W_CopyDialog, "*CopySymbolicLinksButton");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.symlinks=1; else flags.symlinks=0;

  /* one filesystem */
  w_temp=XtNameToWidget(W_CopyDialog, "*CopyOneFilesystemButton");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.one_filesystem=1; else flags.one_filesystem=0;

  XtUnmanageChild(W_CopyDialog);
  FileCopyCommandRoutine(SourceDirectory, CurrentSelectedDirectory, flags);

  AddDirInHistory(CurrentSelectedDirectory);

  XtFree(SourceDirectory);
  XtFree(CurrentSelectedDirectory);
  BlockRefreshRoutine=0;
}

void FileCopyCancelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtFree(SourceDirectory);
  XtFree(CurrentSelectedDirectory);
  FileCopyDeleteDirList();
  XtUnmanageChild(W_CopyDialog);
  BlockRefreshRoutine=0;
}

void FileCopyHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void FileCopyHLinksButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Arg args[2];
  int nbargs;

  nbargs=0;
  XtSetArg(args[nbargs], XmNset, XmUNSET);nbargs++;
  XtSetValues(W_SymLinks, args, nbargs);
}

void FileCopySLinksButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Arg args[2];
  int nbargs;

  nbargs=0;
  XtSetArg(args[nbargs], XmNset, XmUNSET);nbargs++;
  XtSetValues(W_HardLinks, args, nbargs);
}

TCopyDirEntry *FileCopyGetDirEntry(Widget w_search)
{
  TCopyDirEntry *actual;

  actual= CopyDirList;
  while(actual!=NULL && CopyDirIconList.w_icon[actual->icon_index]!=w_search) {
    actual=actual->next;
  }
  return(actual);
}
    
void FileCopySelectDirCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmContainerSelectCallbackStruct *infos;
  TCopyDirEntry *entry;

  infos = (XmContainerSelectCallbackStruct *) widget_data;

  if(infos->reason==XmCR_DEFAULT_ACTION) {
    entry = FileCopyGetDirEntry(infos->selected_items[0]);
    if(entry==NULL) FatalError("filecopy.c","FileCopySelectDirCB","Cannot find corresponding widget");
    if(entry->readable) {
      XtFree(CurrentSelectedDirectory);
      CurrentSelectedDirectory = (char *) XtMalloc(sizeof(char)*(strlen(entry->name)+1));
      strcpy(CurrentSelectedDirectory,entry->name);
      FileCopyCreateDirList();
    }
  }
}

void FileCopyInitDialog()
{
  Widget w_temp;

  Widget w_selectframe, w_separator1, w_okbtn, w_cancelbtn;

  /* Create the dialog */
    
  W_CopyDialog=XmCreateFormDialog(W_MainWindow, "CopyDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("CopyRescanButton", xmPushButtonWidgetClass, W_CopyDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNleftAttachment, XmATTACH_FORM,
				      NULL);

  w_separator1 = XtVaCreateManagedWidget("CopySeparator1", xmSeparatorWidgetClass, W_CopyDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);


  W_Init[1] = XtVaCreateManagedWidget("CopySeparator2", xmSeparatorWidgetClass, W_CopyDialog,
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, w_separator1,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNleftAttachment, XmATTACH_WIDGET,
				      XmNleftWidget, W_Init[0],
				      XmNorientation, XmVERTICAL,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("CopyHelpButton", xmPushButtonWidgetClass, W_CopyDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);

  w_cancelbtn = XtVaCreateManagedWidget("CopyCancelButton", xmPushButtonWidgetClass, W_CopyDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[2],
					NULL);


  w_okbtn = XtVaCreateManagedWidget("CopyOkButton", xmPushButtonWidgetClass, W_CopyDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_WIDGET,
				    XmNleftWidget, W_Init[1],
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
				    NULL);

  /* selector frame */

  w_selectframe = XtVaCreateManagedWidget("CopyDirSelectorFrame", xmFrameWidgetClass, W_CopyDialog, 
					  XmNtopAttachment, XmATTACH_FORM,
					  XmNleftAttachment, XmATTACH_FORM,
					  XmNbottomAttachment, XmATTACH_WIDGET,
					  XmNbottomWidget, w_separator1,
					  XmNrightAttachment, XmATTACH_POSITION,
					  XmNrightPosition, 60,
					  NULL);

  W_Init[0] = XtVaCreateManagedWidget("CopyDirSelectorLabel", xmLabelWidgetClass, w_selectframe, 
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);


  W_Init[1] = XtVaCreateManagedWidget("CopyDirSelectorForm", xmFormWidgetClass, w_selectframe,
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("CopyDirSelectorText", xmComboBoxWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNcomboBoxType, XmDROP_DOWN_COMBO_BOX,
				      NULL);

  UpdateHistoryComboBoxes();

  W_Init[2] = XtVaCreateManagedWidget("CopyDirSelectorListFrame", xmFrameWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[2],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_FORM,
				      NULL);
  
  W_Init[3] = XtVaCreateManagedWidget("CopyDirSelectorListWindow", xmScrolledWindowWidgetClass, W_Init[2],
				      XmNscrollingPolicy, XmAUTOMATIC,
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      NULL);

  W_Init[4] = XtVaCreateManagedWidget("CopyDirSelectorList", xmContainerWidgetClass, W_Init[3],
				      XmNentryViewType, XmSMALL_ICON,
				      XmNlayoutType, XmOUTLINE,
				      XmNselectionPolicy, XmSINGLE_SELECT,
				      NULL);

  /* option box frame */

  W_Init[0] = XtVaCreateManagedWidget("CopyOptionBoxFrame", xmFrameWidgetClass, W_CopyDialog,
				      XmNleftAttachment, XmATTACH_WIDGET,
				      XmNleftWidget, w_selectframe,
				      XmNtopAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("CopyOptionBoxLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("CopyOptionBoxForm", xmFormWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("CopyInteractiveButton", xmToggleButtonWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("CopyRecurseButton", xmToggleButtonWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[2],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("CopyOptionBoxSeparator1", xmSeparatorWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[2],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("CopyPreserveAccessButton", xmToggleButtonWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[2],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("CopyPreserveLinksButton", xmToggleButtonWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[2],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("CopyOptionBoxSeparator2", xmSeparatorWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[2],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("CopyUpdateButton", xmToggleButtonWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[2],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("CopyOptionBoxSeparator3", xmSeparatorWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[2],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("CopyHardLinksButton", xmToggleButtonWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[2],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("CopySymbolicLinksButton", xmToggleButtonWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[2],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("CopyOptionBoxSeparator4", xmSeparatorWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[2],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);
 
  W_Init[2] = XtVaCreateManagedWidget("CopyOneFilesystemButton", xmToggleButtonWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[2],
				      NULL);

  /* hard coded values for the dialog */ 

  XtVaSetValues(W_CopyDialog,
		XmNdefaultButton, w_cancelbtn,
		XmNcancelButton, w_cancelbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);
		
	      
  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_CopyDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);

  /* init dir list */

  CopyDirList=NULL;

  /* Create callbacks */

  /* Rescan Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_CopyDialog, "*CopyRescanButton");
  if(w_temp==NULL) FatalError("filecopy.c","FileCopyInitDialog","Unable to retrieve Rescan Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileCopyRescanButtonCB, NULL);

  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_CopyDialog, "*CopyOkButton");
  if(w_temp==NULL) FatalError("filecopy.c","FileCopyInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileCopyOkButtonCB, NULL);

  /* Cancel Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_CopyDialog, "*CopyCancelButton");
  if(w_temp==NULL) FatalError("filecopy.c","FileCopyInitDialog","Unable to retrieve Cancel Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileCopyCancelButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_CopyDialog, "*CopyHelpButton");
  if(w_temp==NULL) FatalError("filecopy.c","FileCopyInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileCopyHelpButtonCB, NULL);

  /* hard links button */

  W_HardLinks = NULL;
  W_HardLinks = XtNameToWidget(W_CopyDialog, "*CopyHardLinksButton");
  if(W_HardLinks==NULL) FatalError("filecopy.c","FileCopyInitDialog","Unable to retrieve Hard links Button widget");

  XtAddCallback(W_HardLinks, XmNarmCallback, FileCopyHLinksButtonCB, NULL);

  /* symbolic links button */

  W_SymLinks = NULL;
  W_SymLinks = XtNameToWidget(W_CopyDialog, "*CopySymbolicLinksButton");
  if(W_SymLinks==NULL) FatalError("filecopy.c","FileCopyInitDialog","Unable to retrieve Symbolic links Button widget");

  XtAddCallback(W_SymLinks, XmNarmCallback, FileCopySLinksButtonCB, NULL);

  /* Get container & edit text widgets */

  W_CopyTextField = NULL;
  W_CopyTextField = XtNameToWidget(W_CopyDialog, "*CopyDirSelectorText.Text");
  if(W_CopyTextField==NULL) FatalError("filecopy.c","FileCopyInitDialog","Unable to retrieve Select Dir Combo box widget");

  W_CopyContainer = NULL;
  W_CopyContainer = XtNameToWidget(W_CopyDialog, "*CopyDirSelectorList");
  if(W_CopyContainer==NULL) FatalError("filecopy.c","FileCopyInitDialog","Unable to retrieve Dir list Container widget");

  XtAddCallback(W_CopyContainer, XmNdefaultActionCallback, FileCopySelectDirCB, NULL);

}

void FileCopyRoutine(void)
{

  BlockRefreshRoutine=1;
  if(W_CopyDialog==NULL) FileCopyInitDialog();

  SourceDirectory = getcwd(NULL, 0);
  CurrentSelectedDirectory = (char *) XtMalloc(sizeof(char)*(strlen(SourceDirectory)+1));
  strcpy(CurrentSelectedDirectory, SourceDirectory);

  FileCopyCreateDirList();
  XtManageChild(W_CopyDialog);
}

/***************************************************************************
************************ MOVE DIALOG ROUTINE *******************************
****************************************************************************/

int CreateMoveDirIcon(Arg args[], int nbargs)
{
  /* if there is no more pre-allocated buffer */
  if(MoveDirIconList.nb_icons_managed==MoveDirIconList.nb_icons) {
    MoveDirIconList.w_icon= (WidgetList) XtRealloc((char *) MoveDirIconList.w_icon, sizeof(Widget)*MoveDirIconList.nb_icons+1);
    MoveDirIconList.w_icon[MoveDirIconList.nb_icons]= XmCreateIconGadget(W_MoveContainer, "diricon", args, nbargs);
    XtManageChild(MoveDirIconList.w_icon[MoveDirIconList.nb_icons]);
    MoveDirIconList.nb_icons++;
    MoveDirIconList.nb_icons_managed++;
    return(MoveDirIconList.nb_icons-1);
  } else {
    XtSetValues(MoveDirIconList.w_icon[MoveDirIconList.nb_icons_managed], args, nbargs);
    XtManageChild(MoveDirIconList.w_icon[MoveDirIconList.nb_icons_managed]);
    MoveDirIconList.nb_icons_managed++;
    return(MoveDirIconList.nb_icons_managed-1);
  }
}

void FileMoveDeleteDirList(void)
{
  TCopyDirEntry *actual;

  /* Destroy Old Dir List */
  if(CopyDirList!=NULL) {
    actual = CopyDirList;
    while(CopyDirList!=NULL) {
      actual=CopyDirList;
      CopyDirList=CopyDirList->next;
      if(actual->name!=NULL) XtFree((char *) actual->name);
      XtFree((char *) actual);
    }
  }
  XtUnmanageChildren(MoveDirIconList.w_icon, MoveDirIconList.nb_icons_managed);
  MoveDirIconList.nb_icons_managed=0;
  
}

void FileMoveCreateDirList(void)
{
  Arg args[3];
  int nbargs;
  DIR *dir;
  struct dirent *entry;
  TCopyDirEntry *actual;
  struct stat info;
  XmString dirstring;
  int was_a_dir;
  int is_root_dir;
  int i;

  dir = opendir(CurrentSelectedDirectory);
  if(dir==NULL) {
    MessageDialog("Read Error", "Cannot open directory");
    return;
  }

  /* set textfield string */

  nbargs=0;
  XtSetArg(args[nbargs], XmNvalue, CurrentSelectedDirectory);nbargs++;
  XtSetArg(args[nbargs], XmNcursorPosition, strlen(CurrentSelectedDirectory));nbargs++;
  XtSetValues(W_MoveTextField, args, nbargs);

  /* Destroy Old Dir List */
  FileMoveDeleteDirList();

  /* Make New List */
  CopyDirList = (TCopyDirEntry *) XtMalloc(sizeof(TCopyDirEntry));
  actual = CopyDirList;
  actual->next=NULL;
  entry = readdir(dir);
  do {
    actual->name= (char *) XtMalloc(sizeof(char)*(strlen(CurrentSelectedDirectory)+strlen(entry->d_name)+2));
    strcpy(actual->name, CurrentSelectedDirectory);
    if(strcmp(actual->name,"/")==0) is_root_dir=1; else is_root_dir=0;
    if(!is_root_dir && strcmp(entry->d_name,"..")==0) {
      i=strlen(actual->name)-1;
      while(actual->name[i]!='/' && i>0) {
	actual->name[i]=(char)0;
	i--;
      }
      if(i>0) actual->name[i]=(char)0;
    } else {
      if(strcmp(actual->name,"/")!=0) strcat(actual->name, "/");
      strcat(actual->name, entry->d_name);
    }
    stat(actual->name, &info);
    /* if actual entry is a directory */
    was_a_dir=0;
    if(S_ISDIR(info.st_mode) && strcmp(entry->d_name,".")!=0 && ((is_root_dir && strcmp(entry->d_name,"..")!=0) || !is_root_dir)) {
      was_a_dir=1;
      if(access(actual->name, W_OK)==0) actual->writeable=1; else actual->writeable=0;
      if(access(actual->name, R_OK)==0) actual->readable=1; else actual->readable=0;
      /* if user can read and write in directory */
      if(actual->readable && actual->writeable) {
	dirstring = XmStringCreateSimple(entry->d_name);
	if(dirstring==NULL) FatalError("dialogs.c","FileMoveCreateDirList","Cannot create icon string");
	nbargs=0;
	XtSetArg(args[nbargs], XmNsmallIconPixmap, PIX_DIR_closed);nbargs++;
	XtSetArg(args[nbargs], XmNlabelString, dirstring);nbargs++;
	XtSetArg(args[nbargs], XmNentryParent, NULL);nbargs++;
	actual->icon_index=CreateMoveDirIcon(args, nbargs);
	XmStringFree(dirstring);
	/* else if user can't read in directory */
      } else if(!actual->readable) {
	dirstring = XmStringCreateSimple(entry->d_name);
	if(dirstring==NULL) FatalError("dialogs.c","FileMoveCreateDirList","Cannot create icon string");
	nbargs=0;
	XtSetArg(args[nbargs], XmNsmallIconPixmap, PIX_DIR_unreadable);nbargs++;
	XtSetArg(args[nbargs], XmNlabelString, dirstring);nbargs++;
	XtSetArg(args[nbargs], XmNentryParent, NULL);nbargs++;
	actual->icon_index=CreateMoveDirIcon(args, nbargs);
	XmStringFree(dirstring);
      }  else if(!actual->writeable) {
	dirstring = XmStringCreateSimple(entry->d_name);
	if(dirstring==NULL) FatalError("dialogs.c","FileMoveCreateDirList","Cannot create icon string");
	nbargs=0;
	XtSetArg(args[nbargs], XmNsmallIconPixmap, PIX_DIR_read_only);nbargs++;
	XtSetArg(args[nbargs], XmNlabelString, dirstring);nbargs++;
	XtSetArg(args[nbargs], XmNentryParent, NULL);nbargs++;
	actual->icon_index=CreateMoveDirIcon(args, nbargs);
	XmStringFree(dirstring);
      }
    } else {
      XtFree(actual->name);
      actual->name=NULL;
    }
    entry = readdir(dir);
    if(entry!=NULL && was_a_dir) {
      /* Create next node */
      actual->next = (TCopyDirEntry *) XtMalloc(sizeof(TCopyDirEntry));
      actual=actual->next;
      actual->icon_index=-1;
      actual->name=NULL;
      actual->next=NULL;
    }
  } while(entry!=NULL);
  closedir(dir);
}
    

void FileMoveRescanButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtFree(CurrentSelectedDirectory);
  CurrentSelectedDirectory= XmTextGetString(W_MoveTextField);

  FileMoveCreateDirList();
}

void FileMoveOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  TMoveFlags flags;
  Widget w_temp;
  Bool set;

  XtFree(CurrentSelectedDirectory);
  FileMoveDeleteDirList();
  CurrentSelectedDirectory= XmTextGetString(W_MoveTextField);
  if(access(CurrentSelectedDirectory, W_OK)!=0) {
    MessageDialog("write error", "Cannot write destination directory");
    XtUnmanageChild(W_MoveDialog);
    XtFree(CurrentSelectedDirectory);
    XtFree(SourceDirectory);
    return;
  }
  /* get command params */

  /* interactive mode */
  w_temp=XtNameToWidget(W_MoveDialog, "*MoveInteractiveButton");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.interactive=1; else flags.interactive=0;

  /* update files */
  w_temp=XtNameToWidget(W_MoveDialog, "*MoveUpdateButton");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.update=1; else flags.update=0;

  XtUnmanageChild(W_MoveDialog);
  FileMoveCommandRoutine(SourceDirectory, CurrentSelectedDirectory, flags);

  AddDirInHistory(CurrentSelectedDirectory);

  XtFree(SourceDirectory);
  XtFree(CurrentSelectedDirectory);
  BlockRefreshRoutine=0;
}

void FileMoveCancelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtFree(SourceDirectory);
  XtFree(CurrentSelectedDirectory);
  FileMoveDeleteDirList();
  XtUnmanageChild(W_MoveDialog);
  BlockRefreshRoutine=0;
}

void FileMoveHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

TCopyDirEntry *FileMoveGetDirEntry(Widget w_search)
{
  TCopyDirEntry *actual;

  actual= CopyDirList;
  while(actual!=NULL && MoveDirIconList.w_icon[actual->icon_index]!=w_search) {
    actual=actual->next;
  }
  return(actual);
}
    
void FileMoveSelectDirCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XmContainerSelectCallbackStruct *infos;
  TCopyDirEntry *entry;

  infos = (XmContainerSelectCallbackStruct *) widget_data;

  if(infos->reason==XmCR_DEFAULT_ACTION) {
    entry = FileMoveGetDirEntry(infos->selected_items[0]);
    if(entry==NULL) FatalError("filecopy.c","FileMoveSelectDirCB","Cannot find corresponding widget");
    if(entry->readable) {
      XtFree(CurrentSelectedDirectory);
      CurrentSelectedDirectory = (char *) XtMalloc(sizeof(char)*(strlen(entry->name)+1));
      strcpy(CurrentSelectedDirectory,entry->name);
      FileMoveCreateDirList();
    }
  }
}

void FileMoveInitDialog()
{
  Widget w_temp;
  Widget w_selectframe, w_separator1, w_okbtn, w_cancelbtn;

  /* Create the dialog */
    
  W_MoveDialog=XmCreateFormDialog(W_MainWindow, "MoveDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("MoveRescanButton", xmPushButtonWidgetClass, W_MoveDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNleftAttachment, XmATTACH_FORM,
				      NULL);

  w_separator1 = XtVaCreateManagedWidget("MoveSeparator1", xmSeparatorWidgetClass, W_MoveDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);


  W_Init[1] = XtVaCreateManagedWidget("MoveSeparator2", xmSeparatorWidgetClass, W_MoveDialog,
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, w_separator1,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNleftAttachment, XmATTACH_WIDGET,
				      XmNleftWidget, W_Init[0],
				      XmNorientation, XmVERTICAL,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("MoveHelpButton", xmPushButtonWidgetClass, W_MoveDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);

  w_cancelbtn = XtVaCreateManagedWidget("MoveCancelButton", xmPushButtonWidgetClass, W_MoveDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[2],
					NULL);


  w_okbtn = XtVaCreateManagedWidget("MoveOkButton", xmPushButtonWidgetClass, W_MoveDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_WIDGET,
				    XmNleftWidget, W_Init[1],
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
				    NULL);

  /* selector frame */

  w_selectframe = XtVaCreateManagedWidget("MoveDirSelectorFrame", xmFrameWidgetClass, W_MoveDialog, 
					  XmNtopAttachment, XmATTACH_FORM,
					  XmNleftAttachment, XmATTACH_FORM,
					  XmNbottomAttachment, XmATTACH_WIDGET,
					  XmNbottomWidget, w_separator1,
					  XmNrightAttachment, XmATTACH_POSITION,
					  XmNrightPosition, 60,
					  NULL);

  W_Init[0] = XtVaCreateManagedWidget("MoveDirSelectorLabel", xmLabelWidgetClass, w_selectframe, 
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);


  W_Init[1] = XtVaCreateManagedWidget("MoveDirSelectorForm", xmFormWidgetClass, w_selectframe,
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("MoveDirSelectorText", xmComboBoxWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNcomboBoxType, XmDROP_DOWN_COMBO_BOX,
				      NULL);

  UpdateHistoryComboBoxes();

  W_Init[2] = XtVaCreateManagedWidget("MoveDirSelectorListFrame", xmFrameWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[2],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_FORM,
				      NULL);
  
  W_Init[3] = XtVaCreateManagedWidget("MoveDirSelectorListWindow", xmScrolledWindowWidgetClass, W_Init[2],
				      XmNscrollingPolicy, XmAUTOMATIC,
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      NULL);

  W_Init[4] = XtVaCreateManagedWidget("MoveDirSelectorList", xmContainerWidgetClass, W_Init[3],
				      XmNentryViewType, XmSMALL_ICON,
				      XmNlayoutType, XmOUTLINE,
				      XmNselectionPolicy, XmSINGLE_SELECT,
				      NULL);

  /* option box frame */

  W_Init[0] = XtVaCreateManagedWidget("MoveOptionBoxFrame", xmFrameWidgetClass, W_MoveDialog,
				      XmNleftAttachment, XmATTACH_WIDGET,
				      XmNleftWidget, w_selectframe,
				      XmNtopAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("MoveOptionBoxLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("MoveOptionBoxForm", xmFormWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("MoveInteractiveButton", xmToggleButtonWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("MoveOptionBoxSeparator1", xmSeparatorWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[2],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("MoveUpdateButton", xmToggleButtonWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[2],
				      NULL);

  /* hard coded values for the dialog */ 

  XtVaSetValues(W_MoveDialog,
		XmNdefaultButton, w_cancelbtn,
		XmNcancelButton, w_cancelbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);

  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_MoveDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);


  /* init dir list */

  CopyDirList=NULL;

  /* Create callbacks */

  /* Rescan Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_MoveDialog, "*MoveRescanButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileMoveInitDialog","Unable to retrieve Rescan Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileMoveRescanButtonCB, NULL);

  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_MoveDialog, "*MoveOkButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileMoveInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileMoveOkButtonCB, NULL);

  /* Cancel Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_MoveDialog, "*MoveCancelButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileMoveInitDialog","Unable to retrieve Cancel Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileMoveCancelButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_MoveDialog, "*MoveHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileMoveInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileMoveHelpButtonCB, NULL);

  /* Get container & edit text widgets */

  W_MoveTextField = NULL;
  W_MoveTextField = XtNameToWidget(W_MoveDialog, "*MoveDirSelectorText.Text");
  if(W_MoveTextField==NULL) FatalError("dialogs.c","FileMoveInitDialog","Unable to retrieve Select Dir Combo box widget");

  W_MoveContainer = NULL;
  W_MoveContainer = XtNameToWidget(W_MoveDialog, "*MoveDirSelectorList");
  if(W_MoveContainer==NULL) FatalError("dialogs.c","FileMoveInitDialog","Unable to retrieve Dir list Container widget");

  XtAddCallback(W_MoveContainer, XmNdefaultActionCallback, FileMoveSelectDirCB, NULL);
}

void FileMoveRoutine(void)
{

  BlockRefreshRoutine=1;
  if(W_MoveDialog==NULL) FileMoveInitDialog();

  SourceDirectory = getcwd(NULL, 0);
  CurrentSelectedDirectory = (char *) XtMalloc(sizeof(char)*(strlen(SourceDirectory)+1));
  strcpy(CurrentSelectedDirectory, SourceDirectory);

  FileMoveCreateDirList();
  XtManageChild(W_MoveDialog);
}

/***************************************************************************
************************ DELETE DIALOG ROUTINE *****************************
****************************************************************************/

void FileDeleteOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  TDeleteFlags flags;
  Widget w_temp;
  Bool set;

  /* get command params */

  /* interactive mode */
  w_temp=XtNameToWidget(W_DeleteDialog, "*DeleteInteractiveButton");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.interactive=1; else flags.interactive=0;

  /* recurse subdirectories */
  w_temp=XtNameToWidget(W_DeleteDialog, "*DeleteRecurseButton");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.recurse=1; else flags.recurse=0;

  XtUnmanageChild(W_DeleteDialog);
  FileDeleteCommandRoutine(SourceDirectory, flags);
  XtFree(SourceDirectory);
  BlockRefreshRoutine=0;
}

void FileDeleteCancelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtFree(SourceDirectory);
  XtUnmanageChild(W_DeleteDialog);
  BlockRefreshRoutine=0;
}

void FileDeleteHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void FileDeleteInitDialog()
{
  Widget w_temp;
  Widget w_separator1, w_cancelbtn;

  /* Create the dialog */
    
  W_DeleteDialog=XmCreateFormDialog(W_MainWindow, "DeleteDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("DeleteHelpButton", xmPushButtonWidgetClass, W_DeleteDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  w_separator1 = XtVaCreateManagedWidget("DeleteSeparator1", xmSeparatorWidgetClass, W_DeleteDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);



  w_cancelbtn = XtVaCreateManagedWidget("DeleteCancelButton", xmPushButtonWidgetClass, W_DeleteDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[0],
					NULL);


  W_Init[1] = XtVaCreateManagedWidget("DeleteOkButton", xmPushButtonWidgetClass, W_DeleteDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
				    NULL);

  /* Main dialog frame */

  W_Init[0] = XtVaCreateManagedWidget("DeleteOptionBoxFrame", xmFrameWidgetClass, W_DeleteDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("DeleteOptionBoxLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("DeleteOptionBoxForm", xmFormWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("DeleteInteractiveButton", xmToggleButtonWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("DeleteOptionBoxSeparator1", xmSeparatorWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[2],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("DeleteRecurseButton", xmToggleButtonWidgetClass, W_Init[1],
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[2],
				      NULL);

  /* hard coded values for the dialog */ 

  XtVaSetValues(W_DeleteDialog,
		XmNdefaultButton, w_cancelbtn,
		XmNcancelButton, w_cancelbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);

  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_DeleteDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);

  /* Create callbacks */


  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_DeleteDialog, "*DeleteOkButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileDeleteInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileDeleteOkButtonCB, NULL);

  /* Cancel Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_DeleteDialog, "*DeleteCancelButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileDeletInitDialog","Unable to retrieve Cancel Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileDeleteCancelButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_DeleteDialog, "*DeleteHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileDeleteInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileDeleteHelpButtonCB, NULL);

}

void FileDeleteRoutine(void)
{
  if(W_DeleteDialog==NULL) FileDeleteInitDialog();
  BlockRefreshRoutine=1;

  SourceDirectory = getcwd(NULL, 0);

  XtManageChild(W_DeleteDialog);
}

/***************************************************************************
************************ RENAME DIALOG ROUTINE *****************************
****************************************************************************/

void FileRenameOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  char *destfile;

  destfile=XmTextGetString(W_RenameDestTextField);

  XtUnmanageChild(W_RenameDialog);
  if(destfile!=NULL && strcmp(destfile,"")!=0)
    FileRenameCommandRoutine(SourceFile, destfile);
  XtFree(SourceFile);
  if(destfile!=NULL) XtFree(destfile);
  BlockRefreshRoutine=0;
}

void FileRenameCancelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtFree(SourceFile);
  XtUnmanageChild(W_RenameDialog);
  BlockRefreshRoutine=0;
}

void FileRenameHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void FileRenameInitDialog()
{
  Widget w_temp;

  Widget w_separator1, w_cancelbtn;

  /* Create the dialog */
    
  W_RenameDialog=XmCreateFormDialog(W_MainWindow, "RenameDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("RenameHelpButton", xmPushButtonWidgetClass, W_RenameDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  w_separator1 = XtVaCreateManagedWidget("RenameSeparator1", xmSeparatorWidgetClass, W_RenameDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);



  w_cancelbtn = XtVaCreateManagedWidget("RenameCancelButton", xmPushButtonWidgetClass, W_RenameDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[0],
					NULL);


  W_Init[1] = XtVaCreateManagedWidget("RenameOkButton", xmPushButtonWidgetClass, W_RenameDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
 				    NULL);

  /* Dest box frame */

   W_Init[0] = XtVaCreateManagedWidget("RenameDestBoxFrame", xmFrameWidgetClass, W_RenameDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("RenameDestBoxLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("RenameDestText", xmTextFieldWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNeditable, True,
				      NULL);


  /* Source box frame */

  W_Init[0] = XtVaCreateManagedWidget("RenameSourceBoxFrame", xmFrameWidgetClass, W_RenameDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("RenameSourceBoxLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("RenameSourceText", xmTextFieldWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNeditable, False,
				      XmNcursorPositionVisible, False,
				      XmNautoShowCursorPosition, False,
				      NULL);
 

  /* hard coded values for the dialog */ 

  XtVaSetValues(W_RenameDialog,
		XmNdefaultButton, w_cancelbtn,
		XmNcancelButton, w_cancelbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);

  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_RenameDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);


  /* Create callbacks */

  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_RenameDialog, "*RenameOkButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileRenameInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileRenameOkButtonCB, NULL);

  /* Cancel Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_RenameDialog, "*RenameCancelButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileRenameInitDialog","Unable to retrieve Cancel Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileRenameCancelButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_RenameDialog, "*RenameHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileRenameInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileRenameHelpButtonCB, NULL);

  /* Source Text Field */

  W_RenameSourceTextField = NULL;
  W_RenameSourceTextField = XtNameToWidget(W_RenameDialog, "*RenameSourceText");
  if(W_RenameSourceTextField==NULL) FatalError("dialogs.c","FileRenameInitDialog","Unable to retrieve Source file TextField");

  /* Dest Text Field */

  W_RenameDestTextField = NULL;
  W_RenameDestTextField = XtNameToWidget(W_RenameDialog, "*RenameDestText");
  if(W_RenameSourceTextField==NULL) FatalError("dialogs.c","FileRenameInitDialog","Unable to retrieve Dest file TextField");

}

void FileRenameRoutine(void)
{

  if(W_RenameDialog==NULL) FileRenameInitDialog();
  BlockRefreshRoutine=1;

  SourceFile = (char *) XtMalloc(sizeof(char)*(strlen(SelectFileViewList->filenode->file.name)+1));
  strcpy(SourceFile, SelectFileViewList->filenode->file.name);

  XmTextSetString(W_RenameSourceTextField, SourceFile);

  XtManageChild(W_RenameDialog);
}

/***************************************************************************
************************ ATTRIBUTES DIALOG ROUTINE *************************
****************************************************************************/

void FileAttributesOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;
  TAttributesFlags flags;
  Bool set;

  /* Get Attributes Flags */

  /* User */

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesUserRead");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.user_read=1; else flags.user_read=0;

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesUserWrite");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.user_write=1; else flags.user_write=0;

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesUserExec");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.user_exec=1; else flags.user_exec=0;

  /* Group */

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesGroupRead");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.group_read=1; else flags.group_read=0;

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesGroupWrite");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.group_write=1; else flags.group_write=0;

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesGroupExec");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.group_exec=1; else flags.group_exec=0;

  /* Others */

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesOthersRead");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.others_read=1; else flags.others_read=0;

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesOthersWrite");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.others_write=1; else flags.others_write=0;

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesOthersExec");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.others_exec=1; else flags.others_exec=0;

  XtUnmanageChild(W_AttributesDialog);
  FileAttributesCommandRoutine(SourceFile, flags);
  XtFree(SourceFile);
  BlockRefreshRoutine=0;
}

void FileAttributesCancelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtFree(SourceFile);
  XtUnmanageChild(W_AttributesDialog);
  BlockRefreshRoutine=0;
}

void FileAttributesHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void FileAttributesInitDialog()
{
  Widget w_temp;

  Widget w_separator1, w_cancelbtn, w_userframe;

  /* Create the dialog */
    
  W_AttributesDialog=XmCreateFormDialog(W_MainWindow, "AttributesDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("AttributesHelpButton", xmPushButtonWidgetClass, W_AttributesDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  w_separator1 = XtVaCreateManagedWidget("AttributesSeparator3", xmSeparatorWidgetClass, W_AttributesDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);



  w_cancelbtn = XtVaCreateManagedWidget("AttributesCancelButton", xmPushButtonWidgetClass, W_AttributesDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[0],
					NULL);


  W_Init[1] = XtVaCreateManagedWidget("AttributesOkButton", xmPushButtonWidgetClass, W_AttributesDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
 				    NULL);

  /* user frame */

  w_userframe = XtVaCreateManagedWidget("AttributesUserFrame", xmFrameWidgetClass, W_AttributesDialog,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_POSITION,
				      XmNrightPosition, 33,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("AttributesUserLabel", xmLabelWidgetClass, w_userframe,
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("AttributesUserRowCol", xmRowColumnWidgetClass, w_userframe,
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNorientation, XmVERTICAL,
				      XmNpacking, XmPACK_COLUMN,
				      XmNnumColumns, 1,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("AttributesUserRead", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("AttributesUserWrite", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("AttributesUserExec", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);

  /* group frame */

  W_Init[0] = XtVaCreateManagedWidget("AttributesGroupFrame", xmFrameWidgetClass, W_AttributesDialog,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      XmNleftAttachment, XmATTACH_POSITION,
				      XmNleftPosition, 33, 
				      XmNrightAttachment, XmATTACH_POSITION,
				      XmNrightPosition, 66,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("AttributesGroupLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("AttributesGroupRowCol", xmRowColumnWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNorientation, XmVERTICAL,
				      XmNpacking, XmPACK_COLUMN,
				      XmNnumColumns, 1,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("AttributesGroupRead", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("AttributesGroupWrite", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("AttributesGroupExec", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);


  /* others frame */

  W_Init[0] = XtVaCreateManagedWidget("AttributesOthersFrame", xmFrameWidgetClass, W_AttributesDialog,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      XmNleftAttachment, XmATTACH_POSITION,
				      XmNleftPosition, 66, 
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("AttributesOthersLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("AttributesOthersRowCol", xmRowColumnWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNorientation, XmVERTICAL,
				      XmNpacking, XmPACK_COLUMN,
				      XmNnumColumns, 1,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("AttributesOthersRead", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("AttributesOthersWrite", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("AttributesOthersExec", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);

  /* file name frame */

  W_Init[0] = XtVaCreateManagedWidget("AttributesFilenameFrame", xmFrameWidgetClass, W_AttributesDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNtopAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("AttributesFilenameLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("AttributesFilenameText", xmTextFieldWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNeditable, False,
				      XmNcursorPositionVisible, False,
				      NULL);
  
  /* the other parts */

  W_Init[0] = XtVaCreateManagedWidget("AttributesSeparator1", xmSeparatorWidgetClass, W_AttributesDialog,
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[0],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[0] = XtVaCreateManagedWidget("AttributesFileType", xmLabelWidgetClass, W_AttributesDialog,
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[0],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[0] = XtVaCreateManagedWidget("AttributesSeparator2", xmSeparatorWidgetClass, W_AttributesDialog,
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[0],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_userframe,
				      NULL);


  /* hard coded values for the dialog */ 

  XtVaSetValues(W_AttributesDialog,
		XmNdefaultButton, w_cancelbtn,
		XmNcancelButton, w_cancelbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);

  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_AttributesDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);


  /* Create callbacks */

  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_AttributesDialog, "*AttributesOkButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileAttributesInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileAttributesOkButtonCB, NULL);

  /* Cancel Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_AttributesDialog, "*AttributesCancelButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileAttributesInitDialog","Unable to retrieve Cancel Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileAttributesCancelButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_AttributesDialog, "*AttributesHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileAttributesInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileAttributesHelpButtonCB, NULL);

}

void FileAttributesRoutine(void)
{
  Widget w_temp;
  Arg args[2];
  int nbargs;
  XmString typestring;
  char *basictypestr;
  char *spectypestr;
  char *typestr;
  int taille;

  if(W_AttributesDialog==NULL) FileAttributesInitDialog();
  BlockRefreshRoutine=1;

  SourceFile = (char *) XtMalloc(sizeof(char)*(strlen(SelectFileViewList->filenode->file.name)+1));
  strcpy(SourceFile, SelectFileViewList->filenode->file.name);

  w_temp = XtNameToWidget(W_AttributesDialog, "*AttributesFilenameText");
  XmTextSetString(w_temp, SourceFile);

  basictypestr = GetTypeString(SelectFileViewList->filenode->file.type);
  spectypestr= GetFormatDescription(SelectFileViewList->filenode->file.name);

  taille=strlen(basictypestr)+1;
  
  if(spectypestr!=NULL) 
    taille+=6+strlen(spectypestr)+1;

  typestr=XtMalloc(sizeof(char)*taille);
  strcpy(typestr, basictypestr);

  if(spectypestr!=NULL) {
    strcat(typestr, " ( ");
    strcat(typestr, spectypestr);
    strcat(typestr, " ) ");
  }
  
  typestring = XmStringCreateSimple(typestr);

  w_temp = XtNameToWidget(W_AttributesDialog, "*AttributesFileType");
  nbargs=0;
  XtSetArg(args[nbargs], XmNlabelString, typestring);nbargs++;
  XtSetValues(w_temp, args, nbargs);

  XmStringFree(typestring);

  XtFree(typestr);

  /* Init Togglebuttons*/

  /* User */

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesUserRead");
  if(SelectFileViewList->filenode->file.user & READ) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesUserWrite");
  if(SelectFileViewList->filenode->file.user & WRITE) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesUserExec");
  if(SelectFileViewList->filenode->file.user & EXECUTE) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  /* Groups */

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesGroupRead");
  if(SelectFileViewList->filenode->file.group & READ) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesGroupWrite");
  if(SelectFileViewList->filenode->file.group & WRITE) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesGroupExec");
  if(SelectFileViewList->filenode->file.group & EXECUTE) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  /* Others */

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesOthersRead");
  if(SelectFileViewList->filenode->file.others & READ) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesOthersWrite");
  if(SelectFileViewList->filenode->file.others & WRITE) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  w_temp=XtNameToWidget(W_AttributesDialog, "*AttributesOthersExec");
  if(SelectFileViewList->filenode->file.others & EXECUTE) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  XtManageChild(W_AttributesDialog);
}

/***************************************************************************
************************ CREATE DIALOG ROUTINE *****************************
****************************************************************************/

void FileCreateOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;
  Bool makedir;

  char *destfile;

  w_temp = XtNameToWidget(W_CreateDialog, "*CreateText");
  destfile=XmTextGetString(w_temp);

  w_temp = XtNameToWidget(W_CreateDialog, "*CreateToggleButton");
  makedir = XmToggleButtonGetState(w_temp);

  XtUnmanageChild(W_CreateDialog);
  FileCreateCommandRoutine(destfile, makedir);
  if(destfile!=NULL) XtFree(destfile);
  BlockRefreshRoutine=0;
}

void FileCreateCancelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtUnmanageChild(W_CreateDialog);
  BlockRefreshRoutine=0;
}

void FileCreateHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void FileCreateInitDialog()
{
  Widget w_temp;
  Widget w_separator1, w_cancelbtn;

  /* Create the dialog */
    
  W_CreateDialog=XmCreateFormDialog(W_MainWindow, "CreateDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("CreateHelpButton", xmPushButtonWidgetClass, W_CreateDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  w_separator1 = XtVaCreateManagedWidget("CreateSeparator1", xmSeparatorWidgetClass, W_CreateDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);



  w_cancelbtn = XtVaCreateManagedWidget("CreateCancelButton", xmPushButtonWidgetClass, W_CreateDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[0],
					NULL);


  W_Init[1] = XtVaCreateManagedWidget("CreateOkButton", xmPushButtonWidgetClass, W_CreateDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
 				    NULL);

  /* Option box frame */

   W_Init[0] = XtVaCreateManagedWidget("CreateOptionBoxFrame", xmFrameWidgetClass, W_CreateDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("CreateOptionBoxLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("CreateToggleButton", xmToggleButtonWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      NULL);

  /* filename frame */

  W_Init[0] = XtVaCreateManagedWidget("CreateBoxFrame", xmFrameWidgetClass, W_CreateDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, W_Init[0],
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("CreateBoxLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("CreateText", xmTextFieldWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNeditable, True,
				      NULL);
 

  /* hard coded values for the dialog */ 

  XtVaSetValues(W_CreateDialog,
		XmNdefaultButton, w_cancelbtn,
		XmNcancelButton, w_cancelbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);

  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_CreateDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);


  /* Create callbacks */

  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_CreateDialog, "*CreateOkButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileCreateInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileCreateOkButtonCB, NULL);

  /* Cancel Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_CreateDialog, "*CreateCancelButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileCreateInitDialog","Unable to retrieve Cancel Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileCreateCancelButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_CreateDialog, "*CreateHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileCreateInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileCreateHelpButtonCB, NULL);
}

void FileCreateRoutine(void)
{
  if(W_CreateDialog==NULL) FileCreateInitDialog();
  BlockRefreshRoutine=1;
  XtManageChild(W_CreateDialog);
}

/***************************************************************************
**************************** EXEC DIALOG ROUTINE ***************************
****************************************************************************/

void FileExecOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  char *arguments;
  char *program;
  Widget w_temp;
  Bool set;


  /* Get program name */

  w_temp = XtNameToWidget(W_ExecDialog, "*ExecSourceText");
  program = XmTextGetString(w_temp);

  /* Get arguments */
  w_temp = XtNameToWidget(W_ExecDialog, "*ExecArgsBoxText");
  arguments = XmTextGetString(w_temp);

  /* Need an xterm ? */
  w_temp = XtNameToWidget(W_ExecDialog, "*ExecToggleButton");
  set = XmToggleButtonGetState(w_temp);
  
  XtUnmanageChild(W_ExecDialog);
  FileExecCommandRoutine(program, arguments, set);
  if(arguments!=NULL) XtFree(arguments);
  if(program!=NULL) XtFree(program);
  BlockRefreshRoutine=0;
}

void FileExecCancelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtUnmanageChild(W_ExecDialog);
  BlockRefreshRoutine=0;
}

void FileExecHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void FileExecInitDialog()
{
  Widget w_temp;
  Widget w_separator1, w_cancelbtn;

  /* Create the dialog */
    
  W_ExecDialog=XmCreateFormDialog(W_MainWindow, "ExecDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("ExecHelpButton", xmPushButtonWidgetClass, W_ExecDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  w_separator1 = XtVaCreateManagedWidget("ExecSeparator1", xmSeparatorWidgetClass, W_ExecDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);



  w_cancelbtn = XtVaCreateManagedWidget("ExecCancelButton", xmPushButtonWidgetClass, W_ExecDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[0],
					NULL);


  W_Init[1] = XtVaCreateManagedWidget("ExecOkButton", xmPushButtonWidgetClass, W_ExecDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
 				    NULL);

  /* Option box frame */

   W_Init[0] = XtVaCreateManagedWidget("ExecXtermBoxFrame", xmFrameWidgetClass, W_ExecDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("ExecXtermBoxLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("ExecToggleButton", xmToggleButtonWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      NULL);

  /* Args box frame */

  W_Init[0] = XtVaCreateManagedWidget("ExecArgsBoxFrame", xmFrameWidgetClass, W_ExecDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, W_Init[0],
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("ExecArgsBoxLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("ExecArgsBoxText", xmTextFieldWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNeditable, True,
				      NULL);
 
  /* Source box frame */

  W_Init[0] = XtVaCreateManagedWidget("ExecSourceBoxFrame", xmFrameWidgetClass, W_ExecDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, W_Init[0],
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("ExecSourceBoxLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("ExecSourceText", xmTextFieldWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      NULL);
 

  /* hard coded values for the dialog */ 

  XtVaSetValues(W_ExecDialog,
		XmNdefaultButton, w_cancelbtn,
		XmNcancelButton, w_cancelbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);

  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_ExecDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);

  /* Create callbacks */

  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_ExecDialog, "*ExecOkButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileExecInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileExecOkButtonCB, NULL);

  /* Cancel Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_ExecDialog, "*ExecCancelButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileExecInitDialog","Unable to retrieve Cancel Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileExecCancelButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_ExecDialog, "*ExecHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileExecInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileExecHelpButtonCB, NULL);

}

void FileExecRoutine(void)
{
  Widget w_temp;
  TViewNode * actual;


  if(W_ExecDialog==NULL) FileExecInitDialog();
  BlockRefreshRoutine=1;

  w_temp = XtNameToWidget(W_ExecDialog, "*ExecArgsBoxText");
  XmTextSetString(w_temp, "");
  w_temp = XtNameToWidget(W_ExecDialog, "*ExecSourceText");
  XmTextSetString(w_temp, "");

  actual=SelectFileViewList;

  if(actual!=NULL) {
    if(actual->filenode->file.executable) {
      XmTextSetString(w_temp, actual->filenode->file.name);
      actual=actual->next;
    }
    w_temp= XtNameToWidget(W_ExecDialog, "*ExecArgsBoxText");
    while(actual!=NULL) {
      XmTextInsert(w_temp, XmTextGetLastPosition(w_temp), " ");
      XmTextInsert(w_temp, XmTextGetLastPosition(w_temp), actual->filenode->file.name);
      actual=actual->next;
    }
  }

  XtManageChild(W_ExecDialog);
}

void FileDropExecRoutine(void)
{
  Widget w_temp;
  char *string;
  int pos;
  int i;

  if(W_ExecDialog==NULL) FileExecInitDialog();
  BlockRefreshRoutine=1;

  w_temp = XtNameToWidget(W_ExecDialog, "*ExecArgsBoxText");
  XmTextSetString(w_temp, "");

  string=XtMalloc(DropInfos.Size);
  pos=0;
  while(DropInfos.Data[pos]!=(char)0) {
    i=0;
    string[0]=(char)0;
    while(DropInfos.Data[pos]!=(char)0)
      string[i++]=DropInfos.Data[pos++];
    string[i]=(char)0;
    pos++;
    if(string[0]!=(char) 0) {
      XmTextInsert(w_temp, XmTextGetLastPosition(w_temp), string);
      XmTextInsert(w_temp, XmTextGetLastPosition(w_temp), " ");
    }
  }
  XtFree (string);

  w_temp = XtNameToWidget(W_ExecDialog, "*ExecSourceText");
  XmTextSetString(w_temp, DropInfos.DestDir);

  XtFree(DropInfos.Data);
  XtFree(DropInfos.DestDir);
  DropInfos.HandlingDrop=0;

  XtManageChild(W_ExecDialog);
}

/***************************************************************************
*************************** INFO DIALOG ROUTINE ****************************
****************************************************************************/

void FileInfoOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtUnmanageChild(W_InfoDialog);
}

void FileInfoHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void FileInfoInitDialog()
{
  Widget w_temp;
  Widget w_separator1, w_okbtn, w_userframe;

  /* Create the dialog */
    
  W_InfoDialog=XmCreateFormDialog(W_MainWindow, "InfoDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("InfoHelpButton", xmPushButtonWidgetClass, W_InfoDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  w_separator1 = XtVaCreateManagedWidget("InfoSeparator5", xmSeparatorWidgetClass, W_InfoDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);



  w_okbtn = XtVaCreateManagedWidget("InfoOkButton", xmPushButtonWidgetClass, W_InfoDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, W_Init[0],
				    NULL);

  /* user frame */

  w_userframe = XtVaCreateManagedWidget("InfoUserFrame", xmFrameWidgetClass, W_InfoDialog,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_POSITION,
				      XmNrightPosition, 33,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("InfoUserLabel", xmLabelWidgetClass, w_userframe,
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("InfoUserRowCol", xmRowColumnWidgetClass, w_userframe,
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNorientation, XmVERTICAL,
				      XmNpacking, XmPACK_COLUMN,
				      XmNnumColumns, 1,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("InfoUserRead", xmToggleButtonWidgetClass, W_Init[1],
				      XmNsensitive, False,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("InfoUserWrite", xmToggleButtonWidgetClass, W_Init[1],
				      XmNsensitive, False,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("InfoUserExec", xmToggleButtonWidgetClass, W_Init[1],
				      XmNsensitive, False,
				      NULL);

  /* group frame */

  W_Init[0] = XtVaCreateManagedWidget("InfoGroupFrame", xmFrameWidgetClass, W_InfoDialog,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      XmNleftAttachment, XmATTACH_POSITION,
				      XmNleftPosition, 33, 
				      XmNrightAttachment, XmATTACH_POSITION,
				      XmNrightPosition, 66,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("InfoGroupLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("InfoGroupRowCol", xmRowColumnWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNorientation, XmVERTICAL,
				      XmNpacking, XmPACK_COLUMN,
				      XmNnumColumns, 1,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("InfoGroupRead", xmToggleButtonWidgetClass, W_Init[1],
				      XmNsensitive, False,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("InfoGroupWrite", xmToggleButtonWidgetClass, W_Init[1],
				      XmNsensitive, False,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("InfoGroupExec", xmToggleButtonWidgetClass, W_Init[1],
				      XmNsensitive, False,
				      NULL);


  /* others frame */

  W_Init[0] = XtVaCreateManagedWidget("InfoOthersFrame", xmFrameWidgetClass, W_InfoDialog,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      XmNleftAttachment, XmATTACH_POSITION,
				      XmNleftPosition, 66, 
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("InfoOthersLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("InfoOthersRowCol", xmRowColumnWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNorientation, XmVERTICAL,
				      XmNpacking, XmPACK_COLUMN,
				      XmNnumColumns, 1,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("InfoOthersRead", xmToggleButtonWidgetClass, W_Init[1],
				      XmNsensitive, False, 
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("InfoOthersWrite", xmToggleButtonWidgetClass, W_Init[1],
				      XmNsensitive, False,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("InfoOthersExec", xmToggleButtonWidgetClass, W_Init[1],
				      XmNsensitive, False,
				      NULL);

  /* file name frame */

  W_Init[0] = XtVaCreateManagedWidget("InfoFilenameFrame", xmFrameWidgetClass, W_InfoDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNtopAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("InfoFilenameLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("InfoFilenameText", xmTextFieldWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNeditable, False,
				      XmNcursorPositionVisible, False,
				      NULL);
 
  /* the other parts */

  W_Init[0] = XtVaCreateManagedWidget("InfoSeparator1", xmSeparatorWidgetClass, W_InfoDialog,
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[0],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[0] = XtVaCreateManagedWidget("InfoFileType", xmLabelWidgetClass, W_InfoDialog,
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[0],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);

 W_Init[0] = XtVaCreateManagedWidget("InfoSeparator2", xmSeparatorWidgetClass, W_InfoDialog,
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[0],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  W_Init[0] = XtVaCreateManagedWidget("InfoFileSize", xmLabelWidgetClass, W_InfoDialog,
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[0],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);

 W_Init[0] = XtVaCreateManagedWidget("InfoSeparator3", xmSeparatorWidgetClass, W_InfoDialog,
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[0],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[0] = XtVaCreateManagedWidget("InfoFileDateChange", xmLabelWidgetClass, W_InfoDialog,
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[0],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[0] = XtVaCreateManagedWidget("InfoFileDateModif", xmLabelWidgetClass, W_InfoDialog,
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[0],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);

  W_Init[0] = XtVaCreateManagedWidget("InfoFileDateAccess", xmLabelWidgetClass, W_InfoDialog,
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[0],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  W_Init[0] = XtVaCreateManagedWidget("InfoSeparator4", xmSeparatorWidgetClass, W_InfoDialog,
				      XmNtopAttachment, XmATTACH_WIDGET,
				      XmNtopWidget, W_Init[0],
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_userframe,
				      NULL);


  /* hard coded values for the dialog */ 

  XtVaSetValues(W_InfoDialog,
		XmNdefaultButton, w_okbtn,
		XmNcancelButton, w_okbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);

  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_InfoDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);


  /* Create callbacks */

  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_InfoDialog, "*InfoOkButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileInfoInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileInfoOkButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_InfoDialog, "*InfoHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileInfoInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, FileInfoHelpButtonCB, NULL);

}

void FileInfoRoutine(void)
{
  Widget w_temp;
  XmString tempstring;
  char tempstr2[64];
  char tempstr3[128];
  char *basictypestr;
  char *spectypestr;
  char *typestr;
  XmString typestring;
  int taille;
  Arg args[2];
  int nbargs;

  if(W_InfoDialog==NULL) FileInfoInitDialog();

  /* file name */

  SourceFile = (char *) XtMalloc(sizeof(char)*(strlen(SelectFileViewList->filenode->file.name)+1));
  strcpy(SourceFile, SelectFileViewList->filenode->file.name);

  w_temp = XtNameToWidget(W_InfoDialog, "*InfoFilenameText");
  XmTextSetString(w_temp, SourceFile);
  XtFree(SourceFile);

  /* file type */

  basictypestr = GetTypeString(SelectFileViewList->filenode->file.type);
  spectypestr= GetFormatDescription(SelectFileViewList->filenode->file.name);
  taille=strlen(basictypestr)+1;
  
  if(spectypestr!=NULL) 
    taille+=6+strlen(spectypestr)+1;

  typestr=XtMalloc(sizeof(char)*taille);
  strcpy(typestr, basictypestr);

  if(spectypestr!=NULL) {
    strcat(typestr, " ( ");
    strcat(typestr, spectypestr);
    strcat(typestr, " ) ");
  }
  
  typestring = XmStringCreateSimple(typestr);

  w_temp = XtNameToWidget(W_InfoDialog, "*InfoFileType");
  nbargs=0;
  XtSetArg(args[nbargs], XmNlabelString, typestring);nbargs++;
  XtSetValues(w_temp, args, nbargs);

  XmStringFree(typestring);

  XtFree(typestr);

  /* file dates */

  strcpy(tempstr3,"Last Change time : ");
  strftime(tempstr2, sizeof(tempstr2), "%c", localtime(&SelectFileViewList->filenode->file.changetime));
  strcat(tempstr3, tempstr2);
  tempstring = XmStringCreateSimple(tempstr3);
  w_temp = XtNameToWidget(W_InfoDialog, "*InfoFileDateChange");
  XtVaSetValues(w_temp, XmNlabelString, tempstring, NULL);
  XmStringFree(tempstring);

  strcpy(tempstr3,"Last Modification time : ");
  strftime(tempstr2, sizeof(tempstr2), "%c", localtime(&SelectFileViewList->filenode->file.modiftime));
  strcat(tempstr3, tempstr2);
  tempstring = XmStringCreateSimple(tempstr3);
  w_temp = XtNameToWidget(W_InfoDialog, "*InfoFileDateModif");
  XtVaSetValues(w_temp, XmNlabelString, tempstring, NULL);
  XmStringFree(tempstring);

  strcpy(tempstr3,"Last Access time : ");
  strftime(tempstr2, sizeof(tempstr2), "%c", localtime(&SelectFileViewList->filenode->file.accesstime));
  strcat(tempstr3, tempstr2);
  tempstring = XmStringCreateSimple(tempstr3);
  w_temp = XtNameToWidget(W_InfoDialog, "*InfoFileDateAccess");
  XtVaSetValues(w_temp, XmNlabelString, tempstring, NULL);
  XmStringFree(tempstring);

  /* file size */
  if(SelectFileViewList->filenode->file.type == CHAR_DEVICE || SelectFileViewList->filenode->file.type == BLOCK_DEVICE) {
    strcpy(tempstr3, "Special file");
  } else {
    sprintf(tempstr2, "%lu bytes", SelectFileViewList->filenode->file.size);
    strcpy(tempstr3," File Size : ");
    strcat(tempstr3, tempstr2);
  }

  tempstring = XmStringCreateSimple(tempstr3);
  w_temp = XtNameToWidget(W_InfoDialog, "*InfoFileSize");
  XtVaSetValues(w_temp, XmNlabelString, tempstring, NULL);
  XmStringFree(tempstring);

  /* Init Togglebuttons*/

  /* User */

  w_temp=XtNameToWidget(W_InfoDialog, "*InfoUserRead");
  if(SelectFileViewList->filenode->file.user & READ) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  w_temp=XtNameToWidget(W_InfoDialog, "*InfoUserWrite");
  if(SelectFileViewList->filenode->file.user & WRITE) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  w_temp=XtNameToWidget(W_InfoDialog, "*InfoUserExec");
  if(SelectFileViewList->filenode->file.user & EXECUTE) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  /* Groups */

  w_temp=XtNameToWidget(W_InfoDialog, "*InfoGroupRead");
  if(SelectFileViewList->filenode->file.group & READ) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  w_temp=XtNameToWidget(W_InfoDialog, "*InfoGroupWrite");
  if(SelectFileViewList->filenode->file.group & WRITE) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  w_temp=XtNameToWidget(W_InfoDialog, "*InfoGroupExec");
  if(SelectFileViewList->filenode->file.group & EXECUTE) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  /* Others */

  w_temp=XtNameToWidget(W_InfoDialog, "*InfoOthersRead");
  if(SelectFileViewList->filenode->file.others & READ) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  w_temp=XtNameToWidget(W_InfoDialog, "*InfoOthersWrite");
  if(SelectFileViewList->filenode->file.others & WRITE) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  w_temp=XtNameToWidget(W_InfoDialog, "*InfoOthersExec");
  if(SelectFileViewList->filenode->file.others & EXECUTE) XmToggleButtonSetState(w_temp, True, False); else XmToggleButtonSetState(w_temp, False, False);

  XtManageChild(W_InfoDialog);
}

/***************************************************************************
*************************** TAR DIALOG ROUTINE *****************************
****************************************************************************/

void ArchiveTarOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  char *destfile;
  Widget w_temp;
  TTarFlags flags;
  Bool set;
  char *blockstring;

  /* Get Destination file */
  w_temp = XtNameToWidget(W_TarDialog, "*TarFilenameText");
  destfile = XmTextGetString(w_temp);
  
  /* Get Options */
  w_temp = XtNameToWidget(W_TarDialog, "*TarOptionsGZip");
  set = XmToggleButtonGetState(w_temp);
  if(set) flags.gzip=1; else flags.gzip=0;

  w_temp = XtNameToWidget(W_TarDialog, "*TarOptionsAbsolute");
  set = XmToggleButtonGetState(w_temp);
  if(set) flags.absolute=1; else flags.absolute=0;

  w_temp = XtNameToWidget(W_TarDialog, "*TarOptionsDereference");
  set = XmToggleButtonGetState(w_temp);
  if(set) flags.dereference=1; else flags.dereference=0;

  w_temp = XtNameToWidget(W_TarDialog, "*TarOptionsPreserve");
  set = XmToggleButtonGetState(w_temp);
  if(set) flags.preserve=1; else flags.preserve=0;

  w_temp = XtNameToWidget(W_TarDialog, "*TarOptionsOneFilesystem");
  set = XmToggleButtonGetState(w_temp);
  if(set) flags.one_filesystem=1; else flags.one_filesystem=0;

  w_temp = XtNameToWidget(W_TarDialog, "*TarOptionsSetBlock");
  set = XmToggleButtonGetState(w_temp);
  if(set) flags.block_size=1; else flags.block_size=0;

  w_temp = XtNameToWidget(W_TarDialog, "*TarOptionsSetBlockText");
  blockstring = XmTextGetString(w_temp);

  sscanf(blockstring, "%d", &flags.block_size_value);

  XtFree(blockstring);
  
  XtUnmanageChild(W_TarDialog);
  ArchiveTarCommandRoutine(destfile, flags);
  if(destfile!=NULL) XtFree(destfile);
  BlockRefreshRoutine=0;
}

void ArchiveTarCancelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtUnmanageChild(W_TarDialog);
  BlockRefreshRoutine=0;
}

void ArchiveTarHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void ArchiveTarSetBlockButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;
  XmToggleButtonCallbackStruct *infos;

  infos = (XmToggleButtonCallbackStruct *) widget_data;

  w_temp = XtNameToWidget(W_TarDialog, "*TarOptionsSetBlockText");

  if(infos->set == XmSET)     
    XtVaSetValues(w_temp, XmNsensitive, True, XmNeditable, True, XmNcursorPositionVisible, True, NULL, NULL);
  else
    XtVaSetValues(w_temp, XmNsensitive, False, XmNeditable, False, XmNcursorPositionVisible, False, XmNvalue, "20", NULL, NULL);

}

void ArchiveTarInitDialog()
{
  Widget w_temp;
  Widget w_separator1, w_cancelbtn;

  /* Create the dialog */
    
  W_TarDialog=XmCreateFormDialog(W_MainWindow, "TarDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("TarHelpButton", xmPushButtonWidgetClass, W_TarDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  w_separator1 = XtVaCreateManagedWidget("TarSeparator1", xmSeparatorWidgetClass, W_TarDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);



  w_cancelbtn = XtVaCreateManagedWidget("TarCancelButton", xmPushButtonWidgetClass, W_TarDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[0],
					NULL);


  W_Init[1] = XtVaCreateManagedWidget("TarOkButton", xmPushButtonWidgetClass, W_TarDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
 				    NULL);

  /* Option box frame */

   W_Init[0] = XtVaCreateManagedWidget("TarOptionsFrame", xmFrameWidgetClass, W_TarDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("TarOptionsLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);


  W_Init[1] = XtVaCreateManagedWidget("TarOptionsRC", xmRowColumnWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNorientation, XmVERTICAL,
				      XmNpacking, XmPACK_TIGHT,
				      XmNnumColumns, 1,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("TarOptionsGZip", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("TarOptionsSeparator0", xmSeparatorWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("TarOptionsAbsolute", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("TarOptionsSeparator1", xmSeparatorWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("TarOptionsDereference", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("TarOptionsPreserve", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("TarOptionsSeparator2", xmSeparatorWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("TarOptionsOneFilesystem", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("TarOptionsSeparator3", xmSeparatorWidgetClass, W_Init[1],
				      NULL);


  W_Init[2] = XtVaCreateManagedWidget("TarOptionsSetBlockForm",xmFormWidgetClass, W_Init[1],
				      NULL);
  
  W_Init[3] = XtVaCreateManagedWidget("TarOptionsSetBlock", xmToggleButtonWidgetClass, W_Init[2],
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_FORM,
				      NULL);
  
  W_Init[4] = XtVaCreateManagedWidget("TarOptionsSetBlockText", xmTextFieldWidgetClass, W_Init[2],
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNleftAttachment, XmATTACH_WIDGET,
				      XmNleftWidget, W_Init[3],
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNeditable, False,
				      XmNsensitive, False,
				      XmNcursorPositionVisible, False,
				      XmNvalue, "20",
				      NULL);
  
  /* filename frame */

  W_Init[0] = XtVaCreateManagedWidget("TarFilenameFrame", xmFrameWidgetClass, W_TarDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, W_Init[0],
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("TarFilenameLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("TarFilenameText", xmTextFieldWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNeditable, True,
				      NULL);
 

  /* hard coded values for the dialog */ 

  XtVaSetValues(W_TarDialog,
		XmNdefaultButton, w_cancelbtn,
		XmNcancelButton, w_cancelbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);

  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_TarDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);


  /* Create callbacks */

  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_TarDialog, "*TarOkButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveTarInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveTarOkButtonCB, NULL);

  /* Cancel Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_TarDialog, "*TarCancelButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveTarInitDialog","Unable to retrieve Cancel Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveTarCancelButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_TarDialog, "*TarHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveTarInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveTarHelpButtonCB, NULL);

  /* Set Block Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_TarDialog, "*TarOptionsSetBlock");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveTarInitDialog","Unable to retrieve Set Block Button widget");  

  XtAddCallback(w_temp, XmNvalueChangedCallback, ArchiveTarSetBlockButtonCB, NULL);

}

void ArchiveTarRoutine(void)
{
  Widget w_temp;

  if(W_TarDialog==NULL) ArchiveTarInitDialog();
  BlockRefreshRoutine=1;

  w_temp = XtNameToWidget(W_TarDialog, "*TarFilenameText");
  XmTextSetString(w_temp, "");

  XtManageChild(W_TarDialog);
}

/***************************************************************************
*************************** UNTAR DIALOG ROUTINE ***************************
****************************************************************************/

void ArchiveUnTarOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  char *destfile;
  Widget w_temp;
  Bool gunzip;

  /* Get Destination file */
  w_temp = XtNameToWidget(W_UnTarDialog, "*UnTarFilenameText");
  destfile = XmTextGetString(w_temp);

  w_temp = XtNameToWidget(W_UnTarDialog, "*UnTarToggleButton");
  gunzip = XmToggleButtonGetState(w_temp);
  
  XtUnmanageChild(W_UnTarDialog);
  ArchiveUnTarCommandRoutine(destfile, gunzip);
  if(destfile!=NULL) XtFree(destfile);
  BlockRefreshRoutine=0;
}

void ArchiveUnTarCancelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtUnmanageChild(W_UnTarDialog);
  BlockRefreshRoutine=0;
}

void ArchiveUnTarHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void ArchiveUnTarInitDialog()
{
  Widget w_temp;
  Widget w_separator1, w_cancelbtn;

  /* Create the dialog */
    
  W_UnTarDialog=XmCreateFormDialog(W_MainWindow, "UnTarDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("UnTarHelpButton", xmPushButtonWidgetClass, W_UnTarDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  w_separator1 = XtVaCreateManagedWidget("UnTarSeparator1", xmSeparatorWidgetClass, W_UnTarDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);



  w_cancelbtn = XtVaCreateManagedWidget("UnTarCancelButton", xmPushButtonWidgetClass, W_UnTarDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[0],
					NULL);


  W_Init[1] = XtVaCreateManagedWidget("UnTarOkButton", xmPushButtonWidgetClass, W_UnTarDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
 				    NULL);

  /* Option box frame */

   W_Init[0] = XtVaCreateManagedWidget("UnTarOptionBoxFrame", xmFrameWidgetClass, W_UnTarDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("UnTarOptionBoxLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("UnTarToggleButton", xmToggleButtonWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      NULL);

  /* filename frame */

  W_Init[0] = XtVaCreateManagedWidget("UnTarFilenameFrame", xmFrameWidgetClass, W_UnTarDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, W_Init[0],
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("UnTarFilenameLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("UnTarFilenameText", xmTextFieldWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNeditable, False,
				      XmNcursorPositionVisible, False,
				      NULL);
 

  /* hard coded values for the dialog */ 

  XtVaSetValues(W_UnTarDialog,
		XmNdefaultButton, w_cancelbtn,
		XmNcancelButton, w_cancelbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);

  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_UnTarDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);

  /* Create callbacks */

  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_UnTarDialog, "*UnTarOkButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveUnTarInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveUnTarOkButtonCB, NULL);

  /* Cancel Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_UnTarDialog, "*UnTarCancelButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveUnTarInitDialog","Unable to retrieve Cancel Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveUnTarCancelButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_UnTarDialog, "*UnTarHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveUnTarInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveUnTarHelpButtonCB, NULL);

}

void ArchiveUnTarRoutine(void)
{
  Widget w_temp;

  if(W_UnTarDialog==NULL) ArchiveUnTarInitDialog();
  BlockRefreshRoutine=1;

  w_temp = XtNameToWidget(W_UnTarDialog, "*UnTarFilenameText");
  XmTextSetString(w_temp, SelectFileViewList->filenode->file.name);

  XtManageChild(W_UnTarDialog);
}

/***************************************************************************
************************** GZIP DIALOG ROUTINE *****************************
****************************************************************************/

void ArchiveGZipOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  TGZipFlags flags;
  Widget w_temp;
  Bool set;

  /* get command params */

  /* interactive mode */
  w_temp=XtNameToWidget(W_GZipDialog, "*GZipInteractiveButton");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.interactive=1; else flags.interactive=0;

  /* recurse subdirectories */
  w_temp=XtNameToWidget(W_GZipDialog, "*GZipRecurseButton");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.recurse=1; else flags.recurse=0;

  /* quality */
  w_temp=XtNameToWidget(W_GZipDialog, "*GZipQualityScale");
  XmScaleGetValue(w_temp, &flags.quality);

  XtUnmanageChild(W_GZipDialog);
  ArchiveGZipCommandRoutine(flags);
  BlockRefreshRoutine=0;
}

void ArchiveGZipCancelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtUnmanageChild(W_GZipDialog);
  BlockRefreshRoutine=0;
}

void ArchiveGZipHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void ArchiveGZipInitDialog()
{
  Widget w_temp;
  Widget w_separator1, w_cancelbtn;

  /* Create the dialog */
    
  W_GZipDialog=XmCreateFormDialog(W_MainWindow, "GZipDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("GZipHelpButton", xmPushButtonWidgetClass, W_GZipDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  w_separator1 = XtVaCreateManagedWidget("GZipSeparator1", xmSeparatorWidgetClass, W_GZipDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);



  w_cancelbtn = XtVaCreateManagedWidget("GZipCancelButton", xmPushButtonWidgetClass, W_GZipDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[0],
					NULL);


  W_Init[1] = XtVaCreateManagedWidget("GZipOkButton", xmPushButtonWidgetClass, W_GZipDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
				    NULL);

  /* Main dialog frame */

  W_Init[0] = XtVaCreateManagedWidget("GZipOptionBoxFrame", xmFrameWidgetClass, W_GZipDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("GZipOptionBoxLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("GZipOptionBoxRC", xmRowColumnWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNpacking, XmPACK_TIGHT,
				      XmNorientation, XmVERTICAL,
				      XmNnumColumns, 1,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("GZipQualityScale", xmScaleWidgetClass, W_Init[1],
				      XmNminimum, 1,
				      XmNmaximum, 9,
				      XmNorientation, XmHORIZONTAL,
				      XmNvalue, 6,
				      XmNshowArrows, XmEACH_SIDE,
				      XmNshowValue, XmNEAR_SLIDER,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("GZipOptionBoxSeparator1", xmSeparatorWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("GZipInteractiveButton", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("GZipOptionBoxSeparator2", xmSeparatorWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("GZipRecurseButton", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);

  /* hard coded values for the dialog */ 

  XtVaSetValues(W_GZipDialog,
		XmNdefaultButton, w_cancelbtn,
		XmNcancelButton, w_cancelbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);

  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_GZipDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);


  /* Create callbacks */

  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_GZipDialog, "*GZipOkButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveGZipInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveGZipOkButtonCB, NULL);

  /* Cancel Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_GZipDialog, "*GZipCancelButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveGZipInitDialog","Unable to retrieve Cancel Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveGZipCancelButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_GZipDialog, "*GZipHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveGZipInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveGZipHelpButtonCB, NULL);

}

void ArchiveGZipRoutine(void)
{

  if(W_GZipDialog==NULL) ArchiveGZipInitDialog();
  BlockRefreshRoutine=1;

  XtManageChild(W_GZipDialog);
}

/***************************************************************************
************************* GUNZIP DIALOG ROUTINE ****************************
****************************************************************************/

void ArchiveGUnZipOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  TGZipFlags flags;
  Widget w_temp;
  Bool set;

  /* get command params */

  /* interactive mode */
  w_temp=XtNameToWidget(W_GUnZipDialog, "*GUnZipInteractiveButton");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.interactive=1; else flags.interactive=0;

  /* recurse subdirectories */
  w_temp=XtNameToWidget(W_GUnZipDialog, "*GUnZipRecurseButton");
  set=XmToggleButtonGetState(w_temp);
  if(set==True) flags.recurse=1; else flags.recurse=0;

  XtUnmanageChild(W_GUnZipDialog);
  ArchiveGUnZipCommandRoutine(flags);
  BlockRefreshRoutine=0;
}

void ArchiveGUnZipCancelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtUnmanageChild(W_GUnZipDialog);
  BlockRefreshRoutine=0;
}

void ArchiveGUnZipHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void ArchiveGUnZipInitDialog()
{
  Widget w_temp;
  Widget w_separator1, w_cancelbtn;

  /* Create the dialog */
    
  W_GUnZipDialog=XmCreateFormDialog(W_MainWindow, "GUnZipDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("GUnZipHelpButton", xmPushButtonWidgetClass, W_GUnZipDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  w_separator1 = XtVaCreateManagedWidget("GUnZipSeparator1", xmSeparatorWidgetClass, W_GUnZipDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);



  w_cancelbtn = XtVaCreateManagedWidget("GUnZipCancelButton", xmPushButtonWidgetClass, W_GUnZipDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[0],
					NULL);


  W_Init[1] = XtVaCreateManagedWidget("GUnZipOkButton", xmPushButtonWidgetClass, W_GUnZipDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
				    NULL);

  /* Main dialog frame */

  W_Init[0] = XtVaCreateManagedWidget("GUnZipOptionBoxFrame", xmFrameWidgetClass, W_GUnZipDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("GUnZipOptionBoxLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("GUnZipOptionBoxRC", xmRowColumnWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNpacking, XmPACK_TIGHT,
				      XmNorientation, XmVERTICAL,
				      XmNnumColumns, 1,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("GUnZipInteractiveButton", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("GUnZipOptionBoxSeparator1", xmSeparatorWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("GUnZipRecurseButton", xmToggleButtonWidgetClass, W_Init[1],
				      NULL);

  /* hard coded values for the dialog */ 

  XtVaSetValues(W_GUnZipDialog,
		XmNdefaultButton, w_cancelbtn,
		XmNcancelButton, w_cancelbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);

  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_GUnZipDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);

  /* Create callbacks */

  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_GUnZipDialog, "*GUnZipOkButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveGUnZipInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveGUnZipOkButtonCB, NULL);

  /* Cancel Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_GUnZipDialog, "*GUnZipCancelButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveGUnZipInitDialog","Unable to retrieve Cancel Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveGUnZipCancelButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_GUnZipDialog, "*GUnZipHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveGUnZipInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveGUnZipHelpButtonCB, NULL);

}

void ArchiveGUnZipRoutine(void)
{
  if(W_GUnZipDialog==NULL) ArchiveGUnZipInitDialog();
  BlockRefreshRoutine=1;
  XtManageChild(W_GUnZipDialog);
}

/***************************************************************************
************************* ENCODE DIALOG ROUTINE ****************************
****************************************************************************/

void ArchiveEncodeOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;

  char *namefile = NULL;
  char *destfile = NULL;

  w_temp = XtNameToWidget(W_EncodeDialog, "*EncodeNameText");
  namefile = XmTextGetString(w_temp);
  w_temp = XtNameToWidget(W_EncodeDialog, "*EncodeDestText");
  destfile = XmTextGetString(w_temp);
  XtUnmanageChild(W_EncodeDialog);
  ArchiveEncodeCommandRoutine(namefile, destfile);

  if(namefile!=NULL) XtFree(namefile);
  if(destfile!=NULL) XtFree(destfile);

  BlockRefreshRoutine=0;
}

void ArchiveEncodeCancelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtUnmanageChild(W_EncodeDialog);
  BlockRefreshRoutine=0;
}

void ArchiveEncodeHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void ArchiveEncodeInitDialog()
{
  Widget w_temp;
  Widget w_separator1, w_cancelbtn;

  /* Create the dialog */
    
  W_EncodeDialog=XmCreateFormDialog(W_MainWindow, "EncodeDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("EncodeHelpButton", xmPushButtonWidgetClass, W_EncodeDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  w_separator1 = XtVaCreateManagedWidget("EncodeSeparator1", xmSeparatorWidgetClass, W_EncodeDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);



  w_cancelbtn = XtVaCreateManagedWidget("EncodeCancelButton", xmPushButtonWidgetClass, W_EncodeDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[0],
					NULL);


  W_Init[1] = XtVaCreateManagedWidget("EncodeOkButton", xmPushButtonWidgetClass, W_EncodeDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
 				    NULL);

  /* Dest box frame */

   W_Init[0] = XtVaCreateManagedWidget("EncodeDestBoxFrame", xmFrameWidgetClass, W_EncodeDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("EncodeDestBoxLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("EncodeDestText", xmTextFieldWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNeditable, True,
				      NULL);

  /* Args box frame */

  W_Init[0] = XtVaCreateManagedWidget("EncodeNameBoxFrame", xmFrameWidgetClass, W_EncodeDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, W_Init[0],
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("EncodeNameBoxLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("EncodeNameText", xmTextFieldWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNeditable, True,
				      NULL);
 
  /* Source box frame */

  W_Init[0] = XtVaCreateManagedWidget("EncodeSourceBoxFrame", xmFrameWidgetClass, W_EncodeDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, W_Init[0],
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("EncodeSourceBoxLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("EncodeSourceText", xmTextFieldWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNeditable, False,
				      XmNcursorPositionVisible, False,
				      NULL);
 

  /* hard coded values for the dialog */ 

  XtVaSetValues(W_EncodeDialog,
		XmNdefaultButton, w_cancelbtn,
		XmNcancelButton, w_cancelbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);

  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_EncodeDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);

  /* Create callbacks */

  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_EncodeDialog, "*EncodeOkButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveEncodeInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveEncodeOkButtonCB, NULL);

  /* Cancel Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_EncodeDialog, "*EncodeCancelButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveEncodeInitDialog","Unable to retrieve Cancel Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveEncodeCancelButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_EncodeDialog, "*EncodeHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveEncodeInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveEncodeHelpButtonCB, NULL);

}

void ArchiveEncodeRoutine(void)
{
  Widget w_temp;
  
  if(W_EncodeDialog==NULL) ArchiveEncodeInitDialog();
  BlockRefreshRoutine=1;
  
  w_temp = XtNameToWidget(W_EncodeDialog, "*EncodeSourceText");
  XmTextSetString(w_temp, SelectFileViewList->filenode->file.name);

  w_temp = XtNameToWidget(W_EncodeDialog, "*EncodeNameText");
  XmTextSetString(w_temp, SelectFileViewList->filenode->file.name);

  w_temp = XtNameToWidget(W_EncodeDialog, "*EncodeDestText");
  XmTextSetString(w_temp, "");

  XtManageChild(W_EncodeDialog);
}

/***************************************************************************
*************************** DECODE DIALOG ROUTINE ***************************
****************************************************************************/

void ArchiveDecodeOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  char *destfile = NULL;
  Widget w_temp;

  /* Get Destination file */
  w_temp = XtNameToWidget(W_DecodeDialog, "*DecodeFilenameText");
  destfile = XmTextGetString(w_temp);
  
  XtUnmanageChild(W_DecodeDialog);
  ArchiveDecodeCommandRoutine(destfile);
  if(destfile!=NULL) XtFree(destfile);
  BlockRefreshRoutine=0;
}

void ArchiveDecodeCancelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtUnmanageChild(W_DecodeDialog);
  BlockRefreshRoutine=0;
}

void ArchiveDecodeHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void ArchiveDecodeInitDialog()
{
  Widget w_temp;
  Widget w_separator1, w_cancelbtn;

  /* Create the dialog */
    
  W_DecodeDialog=XmCreateFormDialog(W_MainWindow, "DecodeDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("DecodeHelpButton", xmPushButtonWidgetClass, W_DecodeDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  w_separator1 = XtVaCreateManagedWidget("DecodeSeparator1", xmSeparatorWidgetClass, W_DecodeDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);



  w_cancelbtn = XtVaCreateManagedWidget("DecodeCancelButton", xmPushButtonWidgetClass, W_DecodeDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[0],
					NULL);


  W_Init[1] = XtVaCreateManagedWidget("DecodeOkButton", xmPushButtonWidgetClass, W_DecodeDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
 				    NULL);

  /* Filename frame */

   W_Init[0] = XtVaCreateManagedWidget("DecodeFilenameFrame", xmFrameWidgetClass, W_DecodeDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("DecodeFilenameLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("DecodeFilenameText", xmTextFieldWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNeditable, False,
				      XmNcursorPositionVisible, False,
				      NULL);

  /* hard coded values for the dialog */ 

  XtVaSetValues(W_DecodeDialog,
		XmNdefaultButton, w_cancelbtn,
		XmNcancelButton, w_cancelbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);

  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_DecodeDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);

  /* Create callbacks */

  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_DecodeDialog, "*DecodeOkButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveDecodeInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveDecodeOkButtonCB, NULL);

  /* Cancel Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_DecodeDialog, "*DecodeCancelButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveDecodeInitDialog","Unable to retrieve Cancel Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveDecodeCancelButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_DecodeDialog, "*DecodeHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveDecodeInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveDecodeHelpButtonCB, NULL);

}

void ArchiveDecodeRoutine(void)
{
  Widget w_temp;
  
  if(W_DecodeDialog==NULL) ArchiveDecodeInitDialog();
  BlockRefreshRoutine=1;

  w_temp = XtNameToWidget(W_DecodeDialog, "*DecodeFilenameText");
  XmTextSetString(w_temp, SelectFileViewList->filenode->file.name);

  XtManageChild(W_DecodeDialog);
}

/***************************************************************************
*************************** SPLIT DIALOG ROUTINE ***************************
****************************************************************************/

void ArchiveSplitOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;
  TSplitFlags flags;
  Bool set;
  char *tmpstring;

  /* Get Options */
  w_temp = XtNameToWidget(W_SplitDialog, "*SplitOptionsSizeText");
  tmpstring = XmTextGetString(w_temp);
  sscanf(tmpstring, "%d", &flags.size);

  XtFree(tmpstring);

  w_temp = XtNameToWidget(W_SplitDialog, "*SplitOptionsPrefixButton");
  set = XmToggleButtonGetState(w_temp);
  if(set) flags.prefix=1; else flags.prefix=0;

  if(flags.prefix) {
    w_temp = XtNameToWidget(W_SplitDialog, "*SplitOptionsPrefixText");
    flags.prefix_value = XmTextGetString(w_temp);
  }
  
  XtUnmanageChild(W_SplitDialog);
  ArchiveSplitCommandRoutine(flags);
  BlockRefreshRoutine=0;
}

void ArchiveSplitCancelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtUnmanageChild(W_SplitDialog);
  BlockRefreshRoutine=0;
}

void ArchiveSplitHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void ArchiveSplitPrefixButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;
  XmToggleButtonCallbackStruct *infos;

  infos = (XmToggleButtonCallbackStruct *) widget_data;

  w_temp = XtNameToWidget(W_SplitDialog, "*SplitOptionsPrefixText");

  if(infos->set == XmSET)     
    XtVaSetValues(w_temp, XmNsensitive, True, XmNeditable, True, XmNcursorPositionVisible, True, NULL, NULL);
  else
    XtVaSetValues(w_temp, XmNsensitive, False, XmNeditable, False, XmNcursorPositionVisible, False, XmNvalue, "x", NULL, NULL);

}

void ArchiveSplitInitDialog()
{
  Widget w_temp;
  Widget w_separator1, w_cancelbtn;

  /* Create the dialog */
    
  W_SplitDialog=XmCreateFormDialog(W_MainWindow, "SplitDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("SplitHelpButton", xmPushButtonWidgetClass, W_SplitDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  w_separator1 = XtVaCreateManagedWidget("SplitSeparator1", xmSeparatorWidgetClass, W_SplitDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);



  w_cancelbtn = XtVaCreateManagedWidget("SplitCancelButton", xmPushButtonWidgetClass, W_SplitDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[0],
					NULL);


  W_Init[1] = XtVaCreateManagedWidget("SplitOkButton", xmPushButtonWidgetClass, W_SplitDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
 				    NULL);

  /* Options frame */

   W_Init[0] = XtVaCreateManagedWidget("SplitOptionsFrame", xmFrameWidgetClass, W_SplitDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("SplitOptionsLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);


  W_Init[1] = XtVaCreateManagedWidget("SplitOptionsRC", xmRowColumnWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNorientation, XmVERTICAL,
				      XmNpacking, XmPACK_TIGHT,
				      XmNnumColumns, 1,
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("SplitOptionsSizeForm",xmFormWidgetClass, W_Init[1],
				      NULL);
  
  W_Init[3] = XtVaCreateManagedWidget("SplitOptionsSizeLabel", xmLabelWidgetClass, W_Init[2],
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_FORM,
				      NULL);
  
  W_Init[4] = XtVaCreateManagedWidget("SplitOptionsSizeText", xmTextFieldWidgetClass, W_Init[2],
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNleftAttachment, XmATTACH_WIDGET,
				      XmNleftWidget, W_Init[3],
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNeditable, True,
				      XmNvalue, "1440000",
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("SplitOptionsSeparator1",xmSeparatorWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("SplitOptionsPrefixForm",xmFormWidgetClass, W_Init[1],
				      NULL);
  
  W_Init[3] = XtVaCreateManagedWidget("SplitOptionsPrefixButton", xmToggleButtonWidgetClass, W_Init[2],
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_FORM,
				      NULL);
  
  W_Init[4] = XtVaCreateManagedWidget("SplitOptionsPrefixText", xmTextFieldWidgetClass, W_Init[2],
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNleftAttachment, XmATTACH_WIDGET,
				      XmNleftWidget, W_Init[3],
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNeditable, False,
				      XmNvalue, "x",
				      XmNsensitive, False,
				      XmNcursorPositionVisible, False,
				      NULL);
  
  /* filename frame */

  W_Init[0] = XtVaCreateManagedWidget("SplitFilenameFrame", xmFrameWidgetClass, W_SplitDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, W_Init[0],
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("SplitFilenameLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("SplitFilenameText", xmTextFieldWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNeditable, False,
				      XmNcursorPositionVisible, False,
				      NULL);
 

  /* hard coded values for the dialog */ 

  XtVaSetValues(W_SplitDialog,
		XmNdefaultButton, w_cancelbtn,
		XmNcancelButton, w_cancelbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);

  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_SplitDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);

  /* Create callbacks */

  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_SplitDialog, "*SplitOkButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveSplitInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveSplitOkButtonCB, NULL);

  /* Cancel Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_SplitDialog, "*SplitCancelButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveSplitInitDialog","Unable to retrieve Cancel Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveSplitCancelButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_SplitDialog, "*SplitHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveSplitInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveSplitHelpButtonCB, NULL);

  /* Set Block Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_SplitDialog, "*SplitOptionsPrefixButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveSplitInitDialog","Unable to retrieve set prefix Button widget");  

  XtAddCallback(w_temp, XmNvalueChangedCallback, ArchiveSplitPrefixButtonCB, NULL);

}

void ArchiveSplitRoutine(void)
{
  Widget w_temp;

  if(W_SplitDialog==NULL) ArchiveSplitInitDialog();
  BlockRefreshRoutine=1;
  
  w_temp = XtNameToWidget(W_SplitDialog, "*SplitFilenameText");
  XmTextSetString(w_temp, SelectFileViewList->filenode->file.name);

  XtManageChild(W_SplitDialog);
}

/***************************************************************************
************************** UNSPLIT DIALOG ROUTINE **************************
****************************************************************************/

void ArchiveUnSplitOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;
  char *destfile;
  char **sourcefiles;
  XmStringTable stringtable;
  int count, i;

  w_temp = XtNameToWidget(W_UnSplitDialog, "*UnSplitList");  
  XtVaGetValues(w_temp, XmNitems, &stringtable, XmNitemCount, &count, NULL);

  sourcefiles = (char **) XtMalloc(sizeof(char *)*count);
  for(i=0;i<count;i++) 
    XmStringGetLtoR(stringtable[i], XmSTRING_DEFAULT_CHARSET, &sourcefiles[i]);

  w_temp = XtNameToWidget(W_UnSplitDialog, "*UnSplitFilenameText");
  destfile = XmTextGetString(w_temp);

  XtUnmanageChild(W_UnSplitDialog);
  ArchiveUnSplitCommandRoutine(count, sourcefiles, destfile);

  for(i=0;i<count;i++) XtFree(sourcefiles[i]);
  XtFree((char *) sourcefiles);

  if(destfile!=NULL) XtFree(destfile);
  BlockRefreshRoutine=0;
}

void ArchiveUnSplitCancelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtUnmanageChild(W_UnSplitDialog);
  BlockRefreshRoutine=0;
}

void ArchiveUnSplitHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void ArchiveUnSplitOrderUpCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;
  XmString *tempstrtable;
  XmString tempstr;
  int *position;
  int count;

  w_temp = XtNameToWidget(W_UnSplitDialog, "*UnSplitList");  
  XtVaGetValues(w_temp, XmNselectedItemCount, &count, NULL);
  if(count!=0) {
    XtVaGetValues(w_temp, XmNitems, &tempstrtable, XmNselectedPositions, &position, XmNitemCount, &count, NULL);
    if(position[0]>1) {
      tempstr=tempstrtable[position[0]-1];
      tempstrtable[position[0]-1]=tempstrtable[position[0]-2];
      tempstrtable[position[0]-2]=tempstr;
      XtVaSetValues(w_temp, XmNitemCount, count, XmNitems, tempstrtable, NULL);
      XmListSelectPos(w_temp, position[0]-1, True);
      XmListSetPos(w_temp, 1);
    }
  }
}

void ArchiveUnSplitOrderDownCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;
  XmString *tempstrtable;
  XmString tempstr;
  int *position;
  int count;

  w_temp = XtNameToWidget(W_UnSplitDialog, "*UnSplitList");  
  XtVaGetValues(w_temp, XmNselectedItemCount, &count, NULL);
  if(count!=0) {
    XtVaGetValues(w_temp, XmNitems, &tempstrtable, XmNselectedPositions, &position, XmNitemCount, &count, NULL);
    if(position[0]<count) {
      tempstr=tempstrtable[position[0]-1];
      tempstrtable[position[0]-1]=tempstrtable[position[0]];
      tempstrtable[position[0]]=tempstr;
      XtVaSetValues(w_temp, XmNitemCount, count, XmNitems, tempstrtable, NULL);
      XmListSelectPos(w_temp, position[0]+1, True);
      XmListSetPos(w_temp, 1);
    }
  }
}

void ArchiveUnSplitOrderTopCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;
  XmString *tempstrtable;
  XmString tempstr;
  int *position;
  int count, i;

  w_temp = XtNameToWidget(W_UnSplitDialog, "*UnSplitList");  
  XtVaGetValues(w_temp, XmNselectedItemCount, &count, NULL);
  if(count!=0) {
    XtVaGetValues(w_temp, XmNitems, &tempstrtable, XmNselectedPositions, &position, XmNitemCount, &count, NULL);
    if(position[0]>0) {
      tempstr=tempstrtable[position[0]-1];
      for(i=position[0]-1;i>0;i--)
	tempstrtable[i]=tempstrtable[i-1];

      tempstrtable[0]=tempstr;
      XtVaSetValues(w_temp, XmNitemCount, count, XmNitems, tempstrtable, NULL);
      XmListSelectPos(w_temp, 1, True);
      XmListSetPos(w_temp, 1);
    }
  }
}

void ArchiveUnSplitOrderBottomCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;
  XmString *tempstrtable;
  XmString tempstr;
  int *position;
  int count, i;

  w_temp = XtNameToWidget(W_UnSplitDialog, "*UnSplitList");  
  XtVaGetValues(w_temp, XmNselectedItemCount, &count, NULL);
  if(count!=0) {
    XtVaGetValues(w_temp, XmNitems, &tempstrtable, XmNselectedPositions, &position, XmNitemCount, &count, NULL);
    if(position[0]<count) {
      tempstr=tempstrtable[position[0]-1];
      for(i=position[0]-1;i<count-1;i++)
	tempstrtable[i]=tempstrtable[i+1];

      tempstrtable[count-1]=tempstr;
      XtVaSetValues(w_temp, XmNitemCount, count, XmNitems, tempstrtable, NULL);
      XmListSelectPos(w_temp, count, True);
      XmListSetPos(w_temp, 1);
    }
  }
}


void ArchiveUnSplitInitDialog()
{
  Widget w_temp;
  Widget w_separator1, w_cancelbtn;

  /* Create the dialog */
    
  W_UnSplitDialog=XmCreateFormDialog(W_MainWindow, "UnSplitDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("UnSplitHelpButton", xmPushButtonWidgetClass, W_UnSplitDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  w_separator1 = XtVaCreateManagedWidget("UnSplitSeparator1", xmSeparatorWidgetClass, W_UnSplitDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);



  w_cancelbtn = XtVaCreateManagedWidget("UnSplitCancelButton", xmPushButtonWidgetClass, W_UnSplitDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[0],
					NULL);


  W_Init[1] = XtVaCreateManagedWidget("UnSplitOkButton", xmPushButtonWidgetClass, W_UnSplitDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
 				    NULL);

  /* List frame */

   W_Init[0] = XtVaCreateManagedWidget("UnSplitListFrame", xmFrameWidgetClass, W_UnSplitDialog,
				       XmNleftAttachment, XmATTACH_FORM,
				       XmNrightAttachment, XmATTACH_POSITION,
				       XmNrightPosition, 70,
				       XmNtopAttachment, XmATTACH_POSITION,
				       XmNtopPosition, 20,
				       XmNbottomAttachment, XmATTACH_WIDGET,
				       XmNbottomWidget, w_separator1,
				       NULL);
   W_Init[2] = W_Init[0];

  W_Init[1] = XtVaCreateManagedWidget("UnSplitListLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);


  W_Init[1] = XtVaCreateManagedWidget("UnSplitListScrolledW", xmScrolledWindowWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      NULL);
  
  W_Init[1] = XtVaCreateManagedWidget("UnSplitList", xmListWidgetClass, W_Init[1],
				      NULL);

  
  /* filename frame */

  W_Init[0] = XtVaCreateManagedWidget("UnSplitFilenameFrame", xmFrameWidgetClass, W_UnSplitDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNtopAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, W_Init[0],
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("UnSplitFilenameLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("UnSplitFilenameText", xmTextFieldWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNeditable, True,
				      NULL);

  /* Order frame */

   W_Init[0] = XtVaCreateManagedWidget("UnSplitOrderFrame", xmFrameWidgetClass, W_UnSplitDialog,
				       XmNleftAttachment, XmATTACH_WIDGET,
				       XmNleftWidget, W_Init[2],
				       XmNrightAttachment, XmATTACH_FORM,
				       XmNtopAttachment, XmATTACH_WIDGET,
				       XmNtopWidget, W_Init[0],
				       NULL);

  W_Init[1] = XtVaCreateManagedWidget("UnSplitOrderLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);


  W_Init[1] = XtVaCreateManagedWidget("UnSplitOrderRC", xmRowColumnWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNorientation, XmVERTICAL,
				      XmNpacking, XmPACK_TIGHT,
				      XmNnumColumns, 1,
				      NULL);
  
  W_Init[2] = XtVaCreateManagedWidget("UnSplitOrderUp", xmPushButtonWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("UnSplitOrderDown", xmPushButtonWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("UnSplitOrderSeparator1", xmSeparatorWidgetClass, W_Init[1],
				      NULL);

  
  W_Init[2] = XtVaCreateManagedWidget("UnSplitOrderTop", xmPushButtonWidgetClass, W_Init[1],
				      NULL);

  W_Init[2] = XtVaCreateManagedWidget("UnSplitOrderBottom", xmPushButtonWidgetClass, W_Init[1],
				      NULL);

  /* hard coded values for the dialog */ 

  XtVaSetValues(W_UnSplitDialog,
		XmNdefaultButton, w_cancelbtn,
		XmNcancelButton, w_cancelbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);

  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_UnSplitDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);


  /* Create callbacks */

  /* Up Button */
  w_temp = NULL;
  w_temp = XtNameToWidget(W_UnSplitDialog, "*UnSplitOrderUp");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveUnSplitInitDialog","Unable to retrieve Order Up Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveUnSplitOrderUpCB, NULL);

  /* Down Button */
  w_temp = NULL;
  w_temp = XtNameToWidget(W_UnSplitDialog, "*UnSplitOrderDown");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveUnSplitInitDialog","Unable to retrieve Order Down Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveUnSplitOrderDownCB, NULL);

  /* Top Button */
  w_temp = NULL;
  w_temp = XtNameToWidget(W_UnSplitDialog, "*UnSplitOrderTop");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveUnSplitInitDialog","Unable to retrieve Order Top Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveUnSplitOrderTopCB, NULL);

  /* Bottom Button */
  w_temp = NULL;
  w_temp = XtNameToWidget(W_UnSplitDialog, "*UnSplitOrderBottom");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveUnSplitInitDialog","Unable to retrieve Order Bottom Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveUnSplitOrderBottomCB, NULL);


  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_UnSplitDialog, "*UnSplitOkButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveUnSplitInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveUnSplitOkButtonCB, NULL);

  /* Cancel Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_UnSplitDialog, "*UnSplitCancelButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveUnSplitInitDialog","Unable to retrieve Cancel Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveUnSplitCancelButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_UnSplitDialog, "*UnSplitHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","ArchiveUnSplitInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, ArchiveUnSplitHelpButtonCB, NULL);

}

void ArchiveUnSplitRoutine(void)
{
  Widget w_temp;
  TViewNode *actual;
  XmString *items;
  int nbfiles;
  int i;
  
  if(W_UnSplitDialog==NULL) ArchiveUnSplitInitDialog();
  BlockRefreshRoutine=1;

  w_temp = XtNameToWidget(W_UnSplitDialog, "*UnSplitFilenameText");
  XmTextSetString(w_temp, "");

  w_temp = XtNameToWidget(W_UnSplitDialog, "*UnSplitList");

  nbfiles=0;
  actual = SelectFileViewList;
  while(actual!=NULL) {
    nbfiles++;
    actual=actual->next;
  }

  items = (XmString *) XtMalloc(sizeof(XmString)*nbfiles);

  actual = SelectFileViewList;
  for(i=0;i<nbfiles;i++) {
    items[i] = XmStringCreateSimple(actual->filenode->file.name);
    actual=actual->next;
  }
    
  XtVaSetValues(w_temp, XmNitemCount, nbfiles, XmNitems, items, NULL);

  XtManageChild(W_UnSplitDialog);
}

/***************************************************************************
************************ FILTER DIALOG ROUTINE *****************************
****************************************************************************/

void SelectFilterOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;

  char *pattern;

  w_temp = XtNameToWidget(W_SelectFilterDialog, "*FilterText");
  pattern=XmTextGetString(w_temp);

  XtUnmanageChild(W_SelectFilterDialog);
  SelectFilterCommandRoutine(pattern);
  if(pattern!=NULL) XtFree(pattern);
  BlockRefreshRoutine=0;
}

void SelectFilterCancelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtUnmanageChild(W_SelectFilterDialog);
  BlockRefreshRoutine=0;
}

void SelectFilterHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void SelectFilterInitDialog()
{
  Widget w_temp;
  Widget w_separator1, w_cancelbtn, w_okbtn;

  /* Create the dialog */
    
  W_SelectFilterDialog=XmCreateFormDialog(W_MainWindow, "FilterDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("FilterHelpButton", xmPushButtonWidgetClass, W_SelectFilterDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  w_separator1 = XtVaCreateManagedWidget("FilterSeparator1", xmSeparatorWidgetClass, W_SelectFilterDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);



  w_cancelbtn = XtVaCreateManagedWidget("FilterCancelButton", xmPushButtonWidgetClass, W_SelectFilterDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[0],
					NULL);


  w_okbtn = XtVaCreateManagedWidget("FilterOkButton", xmPushButtonWidgetClass, W_SelectFilterDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
 				    NULL);

  /* Filename frame */

   W_Init[0] = XtVaCreateManagedWidget("FilterBoxFrame", xmFrameWidgetClass, W_SelectFilterDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("FilterBoxLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("FilterText", xmTextFieldWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNeditable, True,
				      NULL);

  /* hard coded values for the dialog */ 

  XtVaSetValues(W_SelectFilterDialog,
		XmNdefaultButton, w_okbtn,
		XmNcancelButton, w_cancelbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);

  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_SelectFilterDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);

  /* Create callbacks */

  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_SelectFilterDialog, "*FilterOkButton");
  if(w_temp==NULL) FatalError("dialogs.c","SelectFilterInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, SelectFilterOkButtonCB, NULL);

  /* Cancel Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_SelectFilterDialog, "*FilterCancelButton");
  if(w_temp==NULL) FatalError("dialogs.c","SelectFilterInitDialog","Unable to retrieve Cancel Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, SelectFilterCancelButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_SelectFilterDialog, "*FilterHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","FileCreateInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, SelectFilterHelpButtonCB, NULL);
}

void SelectFilterRoutine(void)
{
  if(W_SelectFilterDialog==NULL) SelectFilterInitDialog();
  BlockRefreshRoutine=1;

  XtManageChild(W_SelectFilterDialog);
}

/***************************************************************************
************************* GO TO DIR DIALOG ROUTINE *************************
****************************************************************************/

void GoToDirOkButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  char *destfile = NULL;
  Widget w_temp;
  Bool state;
  
  /* Get Destination file */
  w_temp = XtNameToWidget(W_GoToDirDialog, "*GoToDirSelectorText.Text");
  destfile = XmTextGetString(w_temp);
  
  XtUnmanageChild(W_GoToDirDialog);

  w_temp=XtNameToWidget(W_GoToDirDialog, "*GoToDirNewWindowButton");
  state=XmToggleButtonGetState(w_temp);

  if(!state) {
    if(GoToDirRefreshRoutine(destfile)!=-1) AddDirInHistory(destfile);
  } else GoToDirNewWindow(destfile);

    if(destfile!=NULL) XtFree(destfile);
    BlockRefreshRoutine=0;
}

void GoToDirCancelButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtUnmanageChild(W_GoToDirDialog);
  BlockRefreshRoutine=0;
}

void GoToDirHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void GoToDirInitDialog()
{
  Widget w_temp;
  Widget w_separator1, w_cancelbtn, w_okbtn;

  /* Create the dialog */
    
  W_GoToDirDialog=XmCreateFormDialog(W_MainWindow, "GoToDirDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("GoToDirHelpButton", xmPushButtonWidgetClass, W_GoToDirDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  w_separator1 = XtVaCreateManagedWidget("GoToDirSeparator1", xmSeparatorWidgetClass, W_GoToDirDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);



  w_cancelbtn = XtVaCreateManagedWidget("GoToDirCancelButton", xmPushButtonWidgetClass, W_GoToDirDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[0],
					NULL);


  w_okbtn = XtVaCreateManagedWidget("GoToDirOkButton", xmPushButtonWidgetClass, W_GoToDirDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
 				    NULL);

  /* Option Frame */

   W_Init[0] = XtVaCreateManagedWidget("GoToDirOptionFrame", xmFrameWidgetClass, W_GoToDirDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, w_separator1,
				      NULL);

   W_Init[1] = XtVaCreateManagedWidget("GoToDirNewWindowButton", xmToggleButtonWidgetClass, W_Init[0],
				       XmNchildType, XmFRAME_WORKAREA_CHILD,
				       XmNset, False,
				       NULL);


  /* Filename frame */

   W_Init[0] = XtVaCreateManagedWidget("GoToDirSelectorFrame", xmFrameWidgetClass, W_GoToDirDialog,
				      XmNleftAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      XmNbottomAttachment, XmATTACH_WIDGET,
				      XmNbottomWidget, W_Init[0],
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("GoToDirSelectorLabel", xmLabelWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_TITLE_CHILD,
				      NULL);

  W_Init[1] = XtVaCreateManagedWidget("GoToDirSelectorText", xmComboBoxWidgetClass, W_Init[0],
				      XmNchildType, XmFRAME_WORKAREA_CHILD,
				      XmNcomboBoxType, XmDROP_DOWN_COMBO_BOX,
				      NULL);

  UpdateHistoryComboBoxes();

  /* hard coded values for the dialog */ 

  XtVaSetValues(W_GoToDirDialog,
		XmNdefaultButton, w_okbtn,
		XmNcancelButton, w_cancelbtn,
		XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		XmNresizePolicy, XmRESIZE_GROW,
		XmNautoUnmanage, False,
		NULL);

  /* Remove decorations for dialogs */

  XtVaSetValues(XtParent(W_GoToDirDialog),
		XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);

  /* Create callbacks */

  /* Ok Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_GoToDirDialog, "*GoToDirOkButton");
  if(w_temp==NULL) FatalError("dialogs.c","GoToDirInitDialog","Unable to retrieve Ok Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, GoToDirOkButtonCB, NULL);

  /* Cancel Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_GoToDirDialog, "*GoToDirCancelButton");
  if(w_temp==NULL) FatalError("dialogs.c","GoToDirInitDialog","Unable to retrieve Cancel Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, GoToDirCancelButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_GoToDirDialog, "*GoToDirHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","GoToDirInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, GoToDirHelpButtonCB, NULL);

}

void GoToDirRoutine(void)
{
  if(W_GoToDirDialog==NULL) GoToDirInitDialog();
  BlockRefreshRoutine=1;

  XtManageChild(W_GoToDirDialog);
}


/***************************************************************************
***************************** DROP DALOG ROUTINE ***************************
****************************************************************************/

void DropYesButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  Widget w_temp;

  w_temp = XtNameToWidget(W_DropDialog, "*DropCopyButton");
  if(XmToggleButtonGetState(w_temp)) DropInfos.DropType=COPY;

  w_temp = XtNameToWidget(W_DropDialog, "*DropMoveButton");
  if(XmToggleButtonGetState(w_temp)) DropInfos.DropType=MOVE;

  w_temp = XtNameToWidget(W_DropDialog, "*DropLinkButton");
  if(XmToggleButtonGetState(w_temp)) DropInfos.DropType=LINK;

  XtUnmanageChild(W_DropDialog);

  DropCommandRoutine();

  XtFree(DropInfos.Data);
  XtFree(DropInfos.DestDir);
  DropInfos.HandlingDrop=0;
  BlockRefreshRoutine=0;

}

void DropNoButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
  XtUnmanageChild(W_DropDialog);
  XtFree(DropInfos.Data);
  XtFree(DropInfos.DestDir);
  DropInfos.HandlingDrop=0;
  BlockRefreshRoutine=0;
}

void DropHelpButtonCB(Widget w_button, XtPointer app_data, XtPointer widget_data)
{
}

void DropInitDialog()
{
  Widget w_temp;
  Widget w_separator1, w_cancelbtn;

  /* Create the dialog */
    
  W_DropDialog=XmCreateFormDialog(W_MainWindow, "DropDialog", NULL, 0);
  

  /* Action zone */

  W_Init[0] = XtVaCreateManagedWidget("DropHelpButton", xmPushButtonWidgetClass, W_DropDialog,
				      XmNbottomAttachment, XmATTACH_FORM,
				      XmNrightAttachment, XmATTACH_FORM,
				      NULL);


  w_separator1 = XtVaCreateManagedWidget("DropSeparator1", xmSeparatorWidgetClass, W_DropDialog,
					 XmNbottomAttachment, XmATTACH_WIDGET,
					 XmNbottomWidget, W_Init[0],
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 NULL);

  w_cancelbtn = XtVaCreateManagedWidget("DropNoButton", xmPushButtonWidgetClass, W_DropDialog,
					XmNbottomAttachment, XmATTACH_FORM,
					XmNrightAttachment, XmATTACH_WIDGET,
					XmNrightWidget, W_Init[0],
					NULL);


  W_Init[1] = XtVaCreateManagedWidget("DropYesButton", xmPushButtonWidgetClass, W_DropDialog,
				    XmNbottomAttachment, XmATTACH_FORM,
				    XmNleftAttachment, XmATTACH_FORM,
				    XmNrightAttachment, XmATTACH_WIDGET,
				    XmNrightWidget, w_cancelbtn,
 				    NULL);

  /* Filename frame */

   W_Init[0] = XtVaCreateManagedWidget("DropOptionFrame", xmFrameWidgetClass, W_DropDialog,
				       XmNtopAttachment, XmATTACH_FORM,
				       XmNbottomAttachment, XmATTACH_WIDGET,
				       XmNbottomWidget, w_separator1,
				       XmNleftAttachment, XmATTACH_FORM,
				       XmNrightAttachment, XmATTACH_FORM,
				       NULL);

   W_Init[1] = XtVaCreateManagedWidget("DropOptionLabel", xmLabelWidgetClass, W_Init[0],
				       XmNchildType, XmFRAME_TITLE_CHILD,
				       NULL);

     W_Init[0] = XtVaCreateManagedWidget("DropOptionRC", xmRowColumnWidgetClass, W_Init[0],
					 XmNchildType, XmFRAME_WORKAREA_CHILD,
					 XmNpacking, XmPACK_TIGHT,
					 XmNnumColumns, 1,
					 XmNradioBehavior, True,
					 NULL);
   
   W_Init[1] = XtVaCreateManagedWidget("DropCopyButton", xmToggleButtonWidgetClass, W_Init[0],
				       XmNindicatorType, XmONE_OF_MANY,
				       NULL);

   W_Init[1] = XtVaCreateManagedWidget("DropMoveButton", xmToggleButtonWidgetClass, W_Init[0],
				       XmNindicatorType, XmONE_OF_MANY,
				       NULL);

   W_Init[1] = XtVaCreateManagedWidget("DropLinkButton", xmToggleButtonWidgetClass, W_Init[0],
				       XmNindicatorType, XmONE_OF_MANY,
				       NULL);
   
   /* hard coded values for the dialog */ 
   
   XtVaSetValues(W_DropDialog,
		 XmNdefaultButton, w_cancelbtn,
		 XmNcancelButton, w_cancelbtn,
		 XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL,
		 XmNresizePolicy, XmRESIZE_GROW,
		 XmNautoUnmanage, False,
		 NULL);

   /* Remove decorations for dialogs */

   XtVaSetValues(XtParent(W_DropDialog),
		 XmNmwmDecorations, MWM_DECOR_ALL | MWM_DECOR_MAXIMIZE | MWM_DECOR_MINIMIZE | MWM_DECOR_MENU,
		NULL);

  /* Create callbacks */

  /* Yes Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_DropDialog, "*DropYesButton");
  if(w_temp==NULL) FatalError("dialogs.c","DropInitDialog","Unable to retrieve Yes Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, DropYesButtonCB, NULL);

  /* No Button */
  
  w_temp = NULL;
  w_temp = XtNameToWidget(W_DropDialog, "*DropNoButton");
  if(w_temp==NULL) FatalError("dialogs.c","DropInitDialog","Unable to retrieve No Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, DropNoButtonCB, NULL);

  /* Help Button */

  w_temp = NULL;
  w_temp = XtNameToWidget(W_DropDialog, "*DropHelpButton");
  if(w_temp==NULL) FatalError("dialogs.c","DropInitDialog","Unable to retrieve Help Button widget");

  XtAddCallback(w_temp, XmNactivateCallback, DropHelpButtonCB, NULL);

}

void DropRoutine(void)
{
  Widget w_temp;
  Bool confirm;

  confirm=True;
  GetVarBool("DndAlwaysConfirm", &confirm);
  if(!confirm) {
    DropCommandRoutine();
    XtFree(DropInfos.Data);
    XtFree(DropInfos.DestDir);
    DropInfos.HandlingDrop=0;
  }

  if(W_DropDialog==NULL) DropInitDialog();
  BlockRefreshRoutine=1;

  w_temp = XtNameToWidget(W_DropDialog, "*DropCopyButton");
  if(DropInfos.DropType==COPY) XmToggleButtonSetState(w_temp, True, True);

  w_temp = XtNameToWidget(W_DropDialog, "*DropMoveButton");
  if(DropInfos.DropType==MOVE) XmToggleButtonSetState(w_temp, True, True);

  w_temp = XtNameToWidget(W_DropDialog, "*DropLinkButton");
  if(DropInfos.DropType==LINK) XmToggleButtonSetState(w_temp, True, True);

  XtManageChild(W_DropDialog);
}

/***************************************************************************
***************************** MODULE INITIALISATION ************************
****************************************************************************/

void InitModuleDialogs(void)
{
  CopyDirIconList.w_icon=NULL;
  CopyDirIconList.nb_icons=0;
  CopyDirIconList.nb_icons_managed=0;

  MoveDirIconList.w_icon=NULL;
  MoveDirIconList.nb_icons=0;
  MoveDirIconList.nb_icons_managed=0;
}
